# This file was auto-generated by Fern from our API Definition.

import typing
from contextlib import asynccontextmanager, contextmanager

import httpx
import websockets.sync.client as websockets_sync_client
from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...core.websocket_compat import InvalidWebSocketStatus, get_status_code
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.query_encoder import single_query_encoder
from ..types.connect_session_settings import ConnectSessionSettings
from .raw_client import AsyncRawChatClient, RawChatClient
from .socket_client import AsyncChatSocketClient, ChatSocketClient

try:
    from websockets.legacy.client import connect as websockets_client_connect  # type: ignore
except ImportError:
    from websockets import connect as websockets_client_connect  # type: ignore


class ChatClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawChatClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawChatClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawChatClient
        """
        return self._raw_client

    @contextmanager
    def connect(
        self,
        *,
        access_token: typing.Optional[str] = None,
        allow_connection: typing.Optional[bool] = None,
        config_id: typing.Optional[str] = None,
        config_version: typing.Optional[int] = None,
        event_limit: typing.Optional[int] = None,
        resumed_chat_group_id: typing.Optional[str] = None,
        verbose_transcription: typing.Optional[bool] = None,
        api_key: typing.Optional[str] = None,
        session_settings: typing.Optional[ConnectSessionSettings] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[ChatSocketClient]:
        """
        Chat with Empathic Voice Interface (EVI)

        Parameters
        ----------
        access_token : typing.Optional[str]
            Access token used for authenticating the client. If not provided, an `api_key` must be provided to authenticate.

            The access token is generated using both an API key and a Secret key, which provides an additional layer of security compared to using just an API key.

            For more details, refer to the [Authentication Strategies Guide](/docs/introduction/api-key#authentication-strategies).

        allow_connection : typing.Optional[bool]
            Allows external connections to this chat via the /connect endpoint.

        config_id : typing.Optional[str]
            The unique identifier for an EVI configuration.

            Include this ID in your connection request to equip EVI with the Prompt, Language Model, Voice, and Tools associated with the specified configuration. If omitted, EVI will apply [default configuration settings](/docs/empathic-voice-interface-evi/configuration#default-configuration).

            For help obtaining this ID, see our [Configuration Guide](/docs/empathic-voice-interface-evi/configuration).

        config_version : typing.Optional[int]
            The version number of the EVI configuration specified by the `config_id`.

            Configs, as well as Prompts and Tools, are versioned. This versioning system supports iterative development, allowing you to progressively refine configurations and revert to previous versions if needed.

            Include this parameter to apply a specific version of an EVI configuration. If omitted, the latest version will be applied.

        event_limit : typing.Optional[int]
            The maximum number of chat events to return from chat history. By default, the system returns up to 300 events (100 events per page × 3 pages). Set this parameter to a smaller value to limit the number of events returned.

        resumed_chat_group_id : typing.Optional[str]
            The unique identifier for a Chat Group. Use this field to preserve context from a previous Chat session.

            A Chat represents a single session from opening to closing a WebSocket connection. In contrast, a Chat Group is a series of resumed Chats that collectively represent a single conversation spanning multiple sessions. Each Chat includes a Chat Group ID, which is used to preserve the context of previous Chat sessions when starting a new one.

            Including the Chat Group ID in the `resumed_chat_group_id` query parameter is useful for seamlessly resuming a Chat after unexpected network disconnections and for picking up conversations exactly where you left off at a later time. This ensures preserved context across multiple sessions.

            There are three ways to obtain the Chat Group ID:

            - [Chat Metadata](/reference/empathic-voice-interface-evi/chat/chat#receive.Chat%20Metadata.type): Upon establishing a WebSocket connection with EVI, the user receives a Chat Metadata message. This message contains a `chat_group_id`, which can be used to resume conversations within this chat group in future sessions.

            - [List Chats endpoint](/reference/empathic-voice-interface-evi/chats/list-chats): Use the GET `/v0/evi/chats` endpoint to obtain the Chat Group ID of individual Chat sessions. This endpoint lists all available Chat sessions and their associated Chat Group ID.

            - [List Chat Groups endpoint](/reference/empathic-voice-interface-evi/chat-groups/list-chat-groups): Use the GET `/v0/evi/chat_groups` endpoint to obtain the Chat Group IDs of all Chat Groups associated with an API key. This endpoint returns a list of all available chat groups.

        verbose_transcription : typing.Optional[bool]
            A flag to enable verbose transcription. Set this query parameter to `"true"` to have unfinalized user transcripts be sent to the client as interim `UserMessage` messages.

        api_key : typing.Optional[str]

        session_settings : ConnectSessionSettings

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChatSocketClient
        """
        ws_url = self._raw_client._client_wrapper.get_environment().evi + "/chat"
        query_params = httpx.QueryParams()
        if access_token is not None:
            query_params = query_params.add("access_token", access_token)
        if allow_connection is not None:
            query_params = query_params.add("allow_connection", allow_connection)
        if config_id is not None:
            query_params = query_params.add("config_id", config_id)
        if config_version is not None:
            query_params = query_params.add("config_version", config_version)
        if event_limit is not None:
            query_params = query_params.add("event_limit", event_limit)
        if resumed_chat_group_id is not None:
            query_params = query_params.add("resumed_chat_group_id", resumed_chat_group_id)
        if verbose_transcription is not None:
            query_params = query_params.add("verbose_transcription", verbose_transcription)
        if api_key is not None:
            query_params = query_params.add("api_key", api_key)
        if session_settings is not None:
            flattened_params = single_query_encoder("session_settings", session_settings)
            for param_key, param_value in flattened_params:
                query_params = query_params.add(param_key, param_value)
        ws_url = ws_url + f"?{query_params}"
        headers = self._raw_client._client_wrapper.get_headers()
        if request_options and "additional_headers" in request_options:
            headers.update(request_options["additional_headers"])
        try:
            with websockets_sync_client.connect(ws_url, additional_headers=headers) as protocol:
                yield ChatSocketClient(websocket=protocol)
        except InvalidWebSocketStatus as exc:
            status_code: int = get_status_code(exc)
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    headers=dict(headers),
                    body="Websocket initialized with invalid credentials.",
                )
            raise ApiError(
                status_code=status_code,
                headers=dict(headers),
                body="Unexpected error when initializing websocket connection.",
            )


class AsyncChatClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawChatClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawChatClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawChatClient
        """
        return self._raw_client

    @asynccontextmanager
    async def connect(
        self,
        *,
        access_token: typing.Optional[str] = None,
        allow_connection: typing.Optional[bool] = None,
        config_id: typing.Optional[str] = None,
        config_version: typing.Optional[int] = None,
        event_limit: typing.Optional[int] = None,
        resumed_chat_group_id: typing.Optional[str] = None,
        verbose_transcription: typing.Optional[bool] = None,
        api_key: typing.Optional[str] = None,
        session_settings: typing.Optional[ConnectSessionSettings] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncChatSocketClient]:
        """
        Chat with Empathic Voice Interface (EVI)

        Parameters
        ----------
        access_token : typing.Optional[str]
            Access token used for authenticating the client. If not provided, an `api_key` must be provided to authenticate.

            The access token is generated using both an API key and a Secret key, which provides an additional layer of security compared to using just an API key.

            For more details, refer to the [Authentication Strategies Guide](/docs/introduction/api-key#authentication-strategies).

        allow_connection : typing.Optional[bool]
            Allows external connections to this chat via the /connect endpoint.

        config_id : typing.Optional[str]
            The unique identifier for an EVI configuration.

            Include this ID in your connection request to equip EVI with the Prompt, Language Model, Voice, and Tools associated with the specified configuration. If omitted, EVI will apply [default configuration settings](/docs/empathic-voice-interface-evi/configuration#default-configuration).

            For help obtaining this ID, see our [Configuration Guide](/docs/empathic-voice-interface-evi/configuration).

        config_version : typing.Optional[int]
            The version number of the EVI configuration specified by the `config_id`.

            Configs, as well as Prompts and Tools, are versioned. This versioning system supports iterative development, allowing you to progressively refine configurations and revert to previous versions if needed.

            Include this parameter to apply a specific version of an EVI configuration. If omitted, the latest version will be applied.

        event_limit : typing.Optional[int]
            The maximum number of chat events to return from chat history. By default, the system returns up to 300 events (100 events per page × 3 pages). Set this parameter to a smaller value to limit the number of events returned.

        resumed_chat_group_id : typing.Optional[str]
            The unique identifier for a Chat Group. Use this field to preserve context from a previous Chat session.

            A Chat represents a single session from opening to closing a WebSocket connection. In contrast, a Chat Group is a series of resumed Chats that collectively represent a single conversation spanning multiple sessions. Each Chat includes a Chat Group ID, which is used to preserve the context of previous Chat sessions when starting a new one.

            Including the Chat Group ID in the `resumed_chat_group_id` query parameter is useful for seamlessly resuming a Chat after unexpected network disconnections and for picking up conversations exactly where you left off at a later time. This ensures preserved context across multiple sessions.

            There are three ways to obtain the Chat Group ID:

            - [Chat Metadata](/reference/empathic-voice-interface-evi/chat/chat#receive.Chat%20Metadata.type): Upon establishing a WebSocket connection with EVI, the user receives a Chat Metadata message. This message contains a `chat_group_id`, which can be used to resume conversations within this chat group in future sessions.

            - [List Chats endpoint](/reference/empathic-voice-interface-evi/chats/list-chats): Use the GET `/v0/evi/chats` endpoint to obtain the Chat Group ID of individual Chat sessions. This endpoint lists all available Chat sessions and their associated Chat Group ID.

            - [List Chat Groups endpoint](/reference/empathic-voice-interface-evi/chat-groups/list-chat-groups): Use the GET `/v0/evi/chat_groups` endpoint to obtain the Chat Group IDs of all Chat Groups associated with an API key. This endpoint returns a list of all available chat groups.

        verbose_transcription : typing.Optional[bool]
            A flag to enable verbose transcription. Set this query parameter to `"true"` to have unfinalized user transcripts be sent to the client as interim `UserMessage` messages.

        api_key : typing.Optional[str]

        session_settings : typing.Optional[ConnectSessionSettings]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncChatSocketClient
        """
        ws_url = self._raw_client._client_wrapper.get_environment().evi + "/chat"
        query_params = httpx.QueryParams()
        if access_token is not None:
            query_params = query_params.add("access_token", access_token)
        if allow_connection is not None:
            query_params = query_params.add("allow_connection", allow_connection)
        if config_id is not None:
            query_params = query_params.add("config_id", config_id)
        if config_version is not None:
            query_params = query_params.add("config_version", config_version)
        if event_limit is not None:
            query_params = query_params.add("event_limit", event_limit)
        if resumed_chat_group_id is not None:
            query_params = query_params.add("resumed_chat_group_id", resumed_chat_group_id)
        if verbose_transcription is not None:
            query_params = query_params.add("verbose_transcription", verbose_transcription)
        if api_key is not None:
            query_params = query_params.add("api_key", api_key)

        if session_settings is not None:
            flattened_params = single_query_encoder("session_settings", session_settings)
            for param_key, param_value in flattened_params:
                query_params = query_params.add(param_key, param_value)

        ws_url = ws_url + f"?{query_params}"
        headers = self._raw_client._client_wrapper.get_headers()
        if request_options and "additional_headers" in request_options:
            headers.update(request_options["additional_headers"])
        try:
            async with websockets_client_connect(ws_url, extra_headers=headers) as protocol:
                yield AsyncChatSocketClient(websocket=protocol)
        except InvalidWebSocketStatus as exc:
            status_code: int = get_status_code(exc)
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    headers=dict(headers),
                    body="Websocket initialized with invalid credentials.",
                )
            raise ApiError(
                status_code=status_code,
                headers=dict(headers),
                body="Unexpected error when initializing websocket connection.",
            )



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .return_config_spec import ReturnConfigSpec


class ReturnChatGroup(UniversalBaseModel):
    """
    A description of chat_group and its status
    """

    active: typing.Optional[bool] = None
    first_start_timestamp: int = pydantic.Field()
    """
    Time at which the first Chat in this Chat Group was created. Measured in seconds since the Unix epoch.
    """

    id: str = pydantic.Field()
    """
    Identifier for the Chat Group. Any Chat resumed from this Chat Group will have the same `chat_group_id`. Formatted as a UUID.
    """

    most_recent_chat_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    The `chat_id` of the most recent Chat in this Chat Group. Formatted as a UUID.
    """

    most_recent_config: typing.Optional[ReturnConfigSpec] = None
    most_recent_start_timestamp: int = pydantic.Field()
    """
    Time at which the most recent Chat in this Chat Group was created. Measured in seconds since the Unix epoch.
    """

    num_chats: int = pydantic.Field()
    """
    The total number of Chats associated with this Chat Group.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .assistant_input import AssistantInput
from .pause_assistant_message import PauseAssistantMessage
from .resume_assistant_message import ResumeAssistantMessage
from .session_settings import SessionSettings
from .tool_error_message import ToolErrorMessage
from .tool_response_message import ToolResponseMessage
from .user_input import UserInput

ControlPlanePublishEvent = typing.Union[
    SessionSettings,
    UserInput,
    AssistantInput,
    ToolResponseMessage,
    ToolErrorMessage,
    PauseAssistantMessage,
    ResumeAssistantMessage,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class PostedUserDefinedToolSpec(UniversalBaseModel):
    """
    A specific tool identifier to be posted to the server
    """

    id: str = pydantic.Field()
    """
    Identifier for a Tool. Formatted as a UUID.
    """

    version: typing.Optional[int] = pydantic.Field(default=None)
    """
    Version number for a Tool. Version numbers should be integers. The combination of configId and version number is unique.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class PostedEllmModel(UniversalBaseModel):
    """
    A eLLM model configuration to be posted to the server
    """

    allow_short_responses: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Boolean indicating if the eLLM is allowed to generate short responses.
    
    If omitted, short responses from the eLLM are enabled by default.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class PostedNudgeSpec(UniversalBaseModel):
    """
    A nudge specification posted to the server
    """

    enabled: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If true, EVI will 'nudge' the user to speak after a determined interval of silence.
    """

    interval_secs: typing.Optional[int] = pydantic.Field(default=None)
    """
    The interval of inactivity (in seconds) before a nudge is triggered.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .posted_utterance_voice_with_id import PostedUtteranceVoiceWithId
from .posted_utterance_voice_with_name import PostedUtteranceVoiceWithName

PostedUtteranceVoice = typing.Union[PostedUtteranceVoiceWithId, PostedUtteranceVoiceWithName]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .posted_webhook_event_type import PostedWebhookEventType


class PostedWebhookSpec(UniversalBaseModel):
    """
    URL and settings for a specific webhook to be posted to the server
    """

    events: typing.List[PostedWebhookEventType] = pydantic.Field()
    """
    The list of events the specified URL is subscribed to. 
    
    See our [webhooks guide](/docs/speech-to-speech-evi/configuration/build-a-configuration#supported-events) for more information on supported events.
    """

    url: str = pydantic.Field()
    """
    The URL where event payloads will be sent. This must be a valid https URL to ensure secure communication. The server at this URL must accept POST requests with a JSON payload.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .types import (
        Alternative,
        Bcp47Tag,
        BoundingBox,
        BurstPrediction,
        Classification,
        CompletedEmbeddingGeneration,
        CompletedInference,
        CompletedState,
        CompletedTlInference,
        CompletedTraining,
        CustomModel,
        CustomModelId,
        CustomModelPrediction,
        CustomModelRequest,
        CustomModelVersionId,
        CustomModelsInferenceJob,
        CustomModelsTrainingJob,
        Dataset,
        DatasetId,
        DatasetVersionId,
        DescriptionsScore,
        Direction,
        EmbeddingGenerationBaseRequest,
        EmbeddingGenerationJob,
        EmotionScore,
        Error,
        EvaluationArgs,
        Face,
        FacePrediction,
        FacemeshPrediction,
        FacsScore,
        Failed,
        FailedState,
        File,
        Granularity,
        GroupedPredictionsBurstPrediction,
        GroupedPredictionsFacePrediction,
        GroupedPredictionsFacemeshPrediction,
        GroupedPredictionsLanguagePrediction,
        GroupedPredictionsNerPrediction,
        GroupedPredictionsProsodyPrediction,
        InProgress,
        InProgressState,
        InferenceBaseRequest,
        InferenceJob,
        InferencePrediction,
        InferenceRequest,
        InferenceResults,
        InferenceSourcePredictResult,
        JobEmbeddingGeneration,
        JobId,
        JobInference,
        JobTlInference,
        JobTraining,
        Language,
        LanguagePrediction,
        Models,
        ModelsPredictions,
        Ner,
        NerPrediction,
        Null,
        PositionInterval,
        PredictionsOptionalNullBurstPrediction,
        PredictionsOptionalNullFacePrediction,
        PredictionsOptionalNullFacemeshPrediction,
        PredictionsOptionalTranscriptionMetadataLanguagePrediction,
        PredictionsOptionalTranscriptionMetadataNerPrediction,
        PredictionsOptionalTranscriptionMetadataProsodyPrediction,
        Prosody,
        ProsodyPrediction,
        Queued,
        QueuedState,
        RegistryFileDetail,
        Regression,
        SentimentScore,
        SortBy,
        Source,
        SourceFile,
        SourceTextSource,
        SourceUrl,
        Source_File,
        Source_Text,
        Source_Url,
        StateEmbeddingGeneration,
        StateEmbeddingGenerationCompletedEmbeddingGeneration,
        StateEmbeddingGenerationFailed,
        StateEmbeddingGenerationInProgress,
        StateEmbeddingGenerationQueued,
        StateEmbeddingGeneration_Completed,
        StateEmbeddingGeneration_Failed,
        StateEmbeddingGeneration_InProgress,
        StateEmbeddingGeneration_Queued,
        StateInference,
        StateInference_Completed,
        StateInference_Failed,
        StateInference_InProgress,
        StateInference_Queued,
        StateTlInference,
        StateTlInferenceCompletedTlInference,
        StateTlInferenceFailed,
        StateTlInferenceInProgress,
        StateTlInferenceQueued,
        StateTlInference_Completed,
        StateTlInference_Failed,
        StateTlInference_InProgress,
        StateTlInference_Queued,
        StateTraining,
        StateTrainingCompletedTraining,
        StateTrainingFailed,
        StateTrainingInProgress,
        StateTrainingQueued,
        StateTraining_Completed,
        StateTraining_Failed,
        StateTraining_InProgress,
        StateTraining_Queued,
        Status,
        Tag,
        Target,
        Task,
        TaskClassification,
        TaskRegression,
        Task_Classification,
        Task_Regression,
        TextSource,
        TimeInterval,
        TlInferenceBaseRequest,
        TlInferencePrediction,
        TlInferenceResults,
        TlInferenceSourcePredictResult,
        ToxicityScore,
        TrainingBaseRequest,
        TrainingCustomModel,
        Transcription,
        TranscriptionMetadata,
        Type,
        Unconfigurable,
        UnionJob,
        UnionPredictResult,
        Url,
        ValidationArgs,
        When,
        Window,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "Alternative": ".types",
    "Bcp47Tag": ".types",
    "BoundingBox": ".types",
    "BurstPrediction": ".types",
    "Classification": ".types",
    "CompletedEmbeddingGeneration": ".types",
    "CompletedInference": ".types",
    "CompletedState": ".types",
    "CompletedTlInference": ".types",
    "CompletedTraining": ".types",
    "CustomModel": ".types",
    "CustomModelId": ".types",
    "CustomModelPrediction": ".types",
    "CustomModelRequest": ".types",
    "CustomModelVersionId": ".types",
    "CustomModelsInferenceJob": ".types",
    "CustomModelsTrainingJob": ".types",
    "Dataset": ".types",
    "DatasetId": ".types",
    "DatasetVersionId": ".types",
    "DescriptionsScore": ".types",
    "Direction": ".types",
    "EmbeddingGenerationBaseRequest": ".types",
    "EmbeddingGenerationJob": ".types",
    "EmotionScore": ".types",
    "Error": ".types",
    "EvaluationArgs": ".types",
    "Face": ".types",
    "FacePrediction": ".types",
    "FacemeshPrediction": ".types",
    "FacsScore": ".types",
    "Failed": ".types",
    "FailedState": ".types",
    "File": ".types",
    "Granularity": ".types",
    "GroupedPredictionsBurstPrediction": ".types",
    "GroupedPredictionsFacePrediction": ".types",
    "GroupedPredictionsFacemeshPrediction": ".types",
    "GroupedPredictionsLanguagePrediction": ".types",
    "GroupedPredictionsNerPrediction": ".types",
    "GroupedPredictionsProsodyPrediction": ".types",
    "InProgress": ".types",
    "InProgressState": ".types",
    "InferenceBaseRequest": ".types",
    "InferenceJob": ".types",
    "InferencePrediction": ".types",
    "InferenceRequest": ".types",
    "InferenceResults": ".types",
    "InferenceSourcePredictResult": ".types",
    "JobEmbeddingGeneration": ".types",
    "JobId": ".types",
    "JobInference": ".types",
    "JobTlInference": ".types",
    "JobTraining": ".types",
    "Language": ".types",
    "LanguagePrediction": ".types",
    "Models": ".types",
    "ModelsPredictions": ".types",
    "Ner": ".types",
    "NerPrediction": ".types",
    "Null": ".types",
    "PositionInterval": ".types",
    "PredictionsOptionalNullBurstPrediction": ".types",
    "PredictionsOptionalNullFacePrediction": ".types",
    "PredictionsOptionalNullFacemeshPrediction": ".types",
    "PredictionsOptionalTranscriptionMetadataLanguagePrediction": ".types",
    "PredictionsOptionalTranscriptionMetadataNerPrediction": ".types",
    "PredictionsOptionalTranscriptionMetadataProsodyPrediction": ".types",
    "Prosody": ".types",
    "ProsodyPrediction": ".types",
    "Queued": ".types",
    "QueuedState": ".types",
    "RegistryFileDetail": ".types",
    "Regression": ".types",
    "SentimentScore": ".types",
    "SortBy": ".types",
    "Source": ".types",
    "SourceFile": ".types",
    "SourceTextSource": ".types",
    "SourceUrl": ".types",
    "Source_File": ".types",
    "Source_Text": ".types",
    "Source_Url": ".types",
    "StateEmbeddingGeneration": ".types",
    "StateEmbeddingGenerationCompletedEmbeddingGeneration": ".types",
    "StateEmbeddingGenerationFailed": ".types",
    "StateEmbeddingGenerationInProgress": ".types",
    "StateEmbeddingGenerationQueued": ".types",
    "StateEmbeddingGeneration_Completed": ".types",
    "StateEmbeddingGeneration_Failed": ".types",
    "StateEmbeddingGeneration_InProgress": ".types",
    "StateEmbeddingGeneration_Queued": ".types",
    "StateInference": ".types",
    "StateInference_Completed": ".types",
    "StateInference_Failed": ".types",
    "StateInference_InProgress": ".types",
    "StateInference_Queued": ".types",
    "StateTlInference": ".types",
    "StateTlInferenceCompletedTlInference": ".types",
    "StateTlInferenceFailed": ".types",
    "StateTlInferenceInProgress": ".types",
    "StateTlInferenceQueued": ".types",
    "StateTlInference_Completed": ".types",
    "StateTlInference_Failed": ".types",
    "StateTlInference_InProgress": ".types",
    "StateTlInference_Queued": ".types",
    "StateTraining": ".types",
    "StateTrainingCompletedTraining": ".types",
    "StateTrainingFailed": ".types",
    "StateTrainingInProgress": ".types",
    "StateTrainingQueued": ".types",
    "StateTraining_Completed": ".types",
    "StateTraining_Failed": ".types",
    "StateTraining_InProgress": ".types",
    "StateTraining_Queued": ".types",
    "Status": ".types",
    "Tag": ".types",
    "Target": ".types",
    "Task": ".types",
    "TaskClassification": ".types",
    "TaskRegression": ".types",
    "Task_Classification": ".types",
    "Task_Regression": ".types",
    "TextSource": ".types",
    "TimeInterval": ".types",
    "TlInferenceBaseRequest": ".types",
    "TlInferencePrediction": ".types",
    "TlInferenceResults": ".types",
    "TlInferenceSourcePredictResult": ".types",
    "ToxicityScore": ".types",
    "TrainingBaseRequest": ".types",
    "TrainingCustomModel": ".types",
    "Transcription": ".types",
    "TranscriptionMetadata": ".types",
    "Type": ".types",
    "Unconfigurable": ".types",
    "UnionJob": ".types",
    "UnionPredictResult": ".types",
    "Url": ".types",
    "ValidationArgs": ".types",
    "When": ".types",
    "Window": ".types",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Alternative",
    "Bcp47Tag",
    "BoundingBox",
    "BurstPrediction",
    "Classification",
    "CompletedEmbeddingGeneration",
    "CompletedInference",
    "CompletedState",
    "CompletedTlInference",
    "CompletedTraining",
    "CustomModel",
    "CustomModelId",
    "CustomModelPrediction",
    "CustomModelRequest",
    "CustomModelVersionId",
    "CustomModelsInferenceJob",
    "CustomModelsTrainingJob",
    "Dataset",
    "DatasetId",
    "DatasetVersionId",
    "DescriptionsScore",
    "Direction",
    "EmbeddingGenerationBaseRequest",
    "EmbeddingGenerationJob",
    "EmotionScore",
    "Error",
    "EvaluationArgs",
    "Face",
    "FacePrediction",
    "FacemeshPrediction",
    "FacsScore",
    "Failed",
    "FailedState",
    "File",
    "Granularity",
    "GroupedPredictionsBurstPrediction",
    "GroupedPredictionsFacePrediction",
    "GroupedPredictionsFacemeshPrediction",
    "GroupedPredictionsLanguagePrediction",
    "GroupedPredictionsNerPrediction",
    "GroupedPredictionsProsodyPrediction",
    "InProgress",
    "InProgressState",
    "InferenceBaseRequest",
    "InferenceJob",
    "InferencePrediction",
    "InferenceRequest",
    "InferenceResults",
    "InferenceSourcePredictResult",
    "JobEmbeddingGeneration",
    "JobId",
    "JobInference",
    "JobTlInference",
    "JobTraining",
    "Language",
    "LanguagePrediction",
    "Models",
    "ModelsPredictions",
    "Ner",
    "NerPrediction",
    "Null",
    "PositionInterval",
    "PredictionsOptionalNullBurstPrediction",
    "PredictionsOptionalNullFacePrediction",
    "PredictionsOptionalNullFacemeshPrediction",
    "PredictionsOptionalTranscriptionMetadataLanguagePrediction",
    "PredictionsOptionalTranscriptionMetadataNerPrediction",
    "PredictionsOptionalTranscriptionMetadataProsodyPrediction",
    "Prosody",
    "ProsodyPrediction",
    "Queued",
    "QueuedState",
    "RegistryFileDetail",
    "Regression",
    "SentimentScore",
    "SortBy",
    "Source",
    "SourceFile",
    "SourceTextSource",
    "SourceUrl",
    "Source_File",
    "Source_Text",
    "Source_Url",
    "StateEmbeddingGeneration",
    "StateEmbeddingGenerationCompletedEmbeddingGeneration",
    "StateEmbeddingGenerationFailed",
    "StateEmbeddingGenerationInProgress",
    "StateEmbeddingGenerationQueued",
    "StateEmbeddingGeneration_Completed",
    "StateEmbeddingGeneration_Failed",
    "StateEmbeddingGeneration_InProgress",
    "StateEmbeddingGeneration_Queued",
    "StateInference",
    "StateInference_Completed",
    "StateInference_Failed",
    "StateInference_InProgress",
    "StateInference_Queued",
    "StateTlInference",
    "StateTlInferenceCompletedTlInference",
    "StateTlInferenceFailed",
    "StateTlInferenceInProgress",
    "StateTlInferenceQueued",
    "StateTlInference_Completed",
    "StateTlInference_Failed",
    "StateTlInference_InProgress",
    "StateTlInference_Queued",
    "StateTraining",
    "StateTrainingCompletedTraining",
    "StateTrainingFailed",
    "StateTrainingInProgress",
    "StateTrainingQueued",
    "StateTraining_Completed",
    "StateTraining_Failed",
    "StateTraining_InProgress",
    "StateTraining_Queued",
    "Status",
    "Tag",
    "Target",
    "Task",
    "TaskClassification",
    "TaskRegression",
    "Task_Classification",
    "Task_Regression",
    "TextSource",
    "TimeInterval",
    "TlInferenceBaseRequest",
    "TlInferencePrediction",
    "TlInferenceResults",
    "TlInferenceSourcePredictResult",
    "ToxicityScore",
    "TrainingBaseRequest",
    "TrainingCustomModel",
    "Transcription",
    "TranscriptionMetadata",
    "Type",
    "Unconfigurable",
    "UnionJob",
    "UnionPredictResult",
    "Url",
    "ValidationArgs",
    "When",
    "Window",
]



# This file was auto-generated by Fern from our API Definition.

"""
Custom Pagination Support

This file is designed to be modified by SDK users to implement their own
pagination logic. The generator will import SyncCustomPager and AsyncCustomPager
from this module when custom pagination is used.

Users should:
1. Implement their custom pager (e.g., PayrocPager, MyCustomPager, etc.)
2. Create adapter classes (SyncCustomPager/AsyncCustomPager) that bridge
   between the generated SDK code and their custom pager implementation
"""

from __future__ import annotations

from typing import Any, AsyncIterator, Generic, Iterator, TypeVar

# Import the base utilities you'll need
# Adjust these imports based on your actual structure
try:
    from .client_wrapper import AsyncClientWrapper, SyncClientWrapper
except ImportError:
    # Fallback for type hints
    AsyncClientWrapper = Any  # type: ignore
    SyncClientWrapper = Any  # type: ignore

TItem = TypeVar("TItem")
TResponse = TypeVar("TResponse")


class SyncCustomPager(Generic[TItem, TResponse]):
    """
    Adapter for custom synchronous pagination.

    The generator will call this with:
        SyncCustomPager(initial_response=response, client_wrapper=client_wrapper)

    Implement this class to extract pagination metadata from your response
    and delegate to your custom pager implementation.

    Example implementation:

        class SyncCustomPager(Generic[TItem, TResponse]):
            def __init__(
                self,
                *,
                initial_response: TResponse,
                client_wrapper: SyncClientWrapper,
            ):
                # Extract data and pagination metadata from response
                data = initial_response.data  # Adjust based on your response structure
                links = initial_response.links

                # Initialize your custom pager
                self._pager = MyCustomPager(
                    current_page=Page(data),
                    httpx_client=client_wrapper.httpx_client,
                    get_headers=client_wrapper.get_headers,
                    # ... other parameters
                )

            def __iter__(self):
                return iter(self._pager)

            # Delegate other methods to your pager...
    """

    def __init__(
        self,
        *,
        initial_response: TResponse,
        client_wrapper: SyncClientWrapper,
    ):
        """
        Initialize the custom pager.

        Args:
            initial_response: The parsed API response from the first request
            client_wrapper: The client wrapper providing HTTP client and utilities
        """
        raise NotImplementedError(
            "SyncCustomPager must be implemented. "
            "Please implement this class in core/custom_pagination.py to define your pagination logic. "
            "See the class docstring for examples."
        )

    def __iter__(self) -> Iterator[TItem]:
        """Iterate through all items across all pages."""
        raise NotImplementedError("Must implement __iter__ method")


class AsyncCustomPager(Generic[TItem, TResponse]):
    """
    Adapter for custom asynchronous pagination.

    The generator will call this with:
        AsyncCustomPager(initial_response=response, client_wrapper=client_wrapper)

    Implement this class to extract pagination metadata from your response
    and delegate to your custom async pager implementation.

    Example implementation:

        class AsyncCustomPager(Generic[TItem, TResponse]):
            def __init__(
                self,
                *,
                initial_response: TResponse,
                client_wrapper: AsyncClientWrapper,
            ):
                # Extract data and pagination metadata from response
                data = initial_response.data  # Adjust based on your response structure
                links = initial_response.links

                # Initialize your custom async pager
                self._pager = MyAsyncCustomPager(
                    current_page=Page(data),
                    httpx_client=client_wrapper.httpx_client,
                    get_headers=client_wrapper.get_headers,
                    # ... other parameters
                )

            async def __aiter__(self):
                return self._pager.__aiter__()

            # Delegate other methods to your pager...
    """

    def __init__(
        self,
        *,
        initial_response: TResponse,
        client_wrapper: AsyncClientWrapper,
    ):
        """
        Initialize the custom async pager.

        Args:
            initial_response: The parsed API response from the first request
            client_wrapper: The client wrapper providing HTTP client and utilities
        """
        raise NotImplementedError(
            "AsyncCustomPager must be implemented. "
            "Please implement this class in core/custom_pagination.py to define your pagination logic. "
            "See the class docstring for examples."
        )

    async def __aiter__(self) -> AsyncIterator[TItem]:
        """Asynchronously iterate through all items across all pages."""
        raise NotImplementedError("Must implement __aiter__ method")



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ReturnPrompt(UniversalBaseModel):
    """
    A specific prompt version returned from the server
    """

    created_on: int = pydantic.Field()
    """
    Time at which the Prompt was created. Measured in seconds since the Unix epoch.
    """

    id: str = pydantic.Field()
    """
    Identifier for a Prompt. Formatted as a UUID.
    """

    modified_on: int = pydantic.Field()
    """
    Time at which the Prompt was last modified. Measured in seconds since the Unix epoch.
    """

    name: str = pydantic.Field()
    """
    Name applied to all versions of a particular Prompt.
    """

    text: str = pydantic.Field()
    """
    Instructions used to shape EVI's behavior, responses, and style.
    
    You can use the Prompt to define a specific goal or role for EVI, specifying how it should act or what it should focus on during the conversation. For example, EVI can be instructed to act as a customer support representative, a fitness coach, or a travel advisor, each with its own set of behaviors and response styles. For help writing a system prompt, see our [Prompting Guide](/docs/speech-to-speech-evi/guides/prompting).
    """

    version: int = pydantic.Field()
    """
    Version number for a Prompt. Version numbers should be integers. The combination of configId and version number is unique.
    """

    version_description: typing.Optional[str] = pydantic.Field(default=None)
    """
    An optional description of the Prompt version.
    """

    version_type: str = pydantic.Field()
    """
    Indicates whether this prompt is using a fixed version number or auto-updating to the latest version. Values from the VersionType enum.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from contextlib import asynccontextmanager, contextmanager

import websockets.sync.client as websockets_sync_client
from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.query_encoder import encode_query
from ...core.remove_none_from_dict import remove_none_from_dict
from ...core.request_options import RequestOptions
from ...core.websocket_compat import InvalidWebSocketStatus, get_status_code
from ..types.control_plane_publish_event import ControlPlanePublishEvent
from .raw_client import AsyncRawControlPlaneClient, RawControlPlaneClient
from .socket_client import AsyncControlPlaneSocketClient, ControlPlaneSocketClient

try:
    from websockets.legacy.client import connect as websockets_client_connect  # type: ignore
except ImportError:
    from websockets import connect as websockets_client_connect  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ControlPlaneClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawControlPlaneClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawControlPlaneClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawControlPlaneClient
        """
        return self._raw_client

    def send(
        self,
        chat_id: str,
        *,
        request: ControlPlanePublishEvent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Send a message to a specific chat.

        Parameters
        ----------
        chat_id : str

        request : ControlPlanePublishEvent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from hume import HumeClient
        from hume.empathic_voice import SessionSettings

        client = HumeClient(
            api_key="YOUR_API_KEY",
        )
        client.empathic_voice.control_plane.send(
            chat_id="chat_id",
            request=SessionSettings(),
        )
        """
        _response = self._raw_client.send(chat_id, request=request, request_options=request_options)
        return _response.data

    @contextmanager
    def connect(
        self,
        chat_id: str,
        *,
        access_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[ControlPlaneSocketClient]:
        """
        Connects to an in-progress EVI chat session. The original chat must have been started with `allow_connection=true`. The connection can be used to send and receive the same messages as the original chat, with the exception that `audio_input` messages are not allowed.

        Parameters
        ----------
        chat_id : str
            The ID of the chat to connect to.

        access_token : typing.Optional[str]
            Access token used for authenticating the client. If not provided, an `api_key` must be provided to authenticate.

            The access token is generated using both an API key and a Secret key, which provides an additional layer of security compared to using just an API key.

            For more details, refer to the [Authentication Strategies Guide](/docs/introduction/api-key#authentication-strategies).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ControlPlaneSocketClient
        """
        ws_url = self._raw_client._client_wrapper.get_environment().evi + "/chat/"
        _encoded_query_params = encode_query(
            jsonable_encoder(
                remove_none_from_dict(
                    {
                        "access_token": access_token,
                        **(
                            request_options.get("additional_query_parameters", {}) or {}
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            )
        )
        if _encoded_query_params:
            ws_url = ws_url + "?" + urllib.parse.urlencode(_encoded_query_params)
        headers = self._raw_client._client_wrapper.get_headers()
        if request_options and "additional_headers" in request_options:
            headers.update(request_options["additional_headers"])
        try:
            with websockets_sync_client.connect(ws_url, additional_headers=headers) as protocol:
                yield ControlPlaneSocketClient(websocket=protocol)
        except InvalidWebSocketStatus as exc:
            status_code: int = get_status_code(exc)
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    headers=dict(headers),
                    body="Websocket initialized with invalid credentials.",
                )
            raise ApiError(
                status_code=status_code,
                headers=dict(headers),
                body="Unexpected error when initializing websocket connection.",
            )


class AsyncControlPlaneClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawControlPlaneClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawControlPlaneClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawControlPlaneClient
        """
        return self._raw_client

    async def send(
        self,
        chat_id: str,
        *,
        request: ControlPlanePublishEvent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Send a message to a specific chat.

        Parameters
        ----------
        chat_id : str

        request : ControlPlanePublishEvent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from hume import AsyncHumeClient
        from hume.empathic_voice import SessionSettings

        client = AsyncHumeClient(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.empathic_voice.control_plane.send(
                chat_id="chat_id",
                request=SessionSettings(),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.send(chat_id, request=request, request_options=request_options)
        return _response.data

    @asynccontextmanager
    async def connect(
        self,
        chat_id: str,
        *,
        access_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncControlPlaneSocketClient]:
        """
        Connects to an in-progress EVI chat session. The original chat must have been started with `allow_connection=true`. The connection can be used to send and receive the same messages as the original chat, with the exception that `audio_input` messages are not allowed.

        Parameters
        ----------
        chat_id : str
            The ID of the chat to connect to.

        access_token : typing.Optional[str]
            Access token used for authenticating the client. If not provided, an `api_key` must be provided to authenticate.

            The access token is generated using both an API key and a Secret key, which provides an additional layer of security compared to using just an API key.

            For more details, refer to the [Authentication Strategies Guide](/docs/introduction/api-key#authentication-strategies).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncControlPlaneSocketClient
        """
        ws_url = self._raw_client._client_wrapper.get_environment().evi + "/chat/"
        _encoded_query_params = encode_query(
            jsonable_encoder(
                remove_none_from_dict(
                    {
                        "access_token": access_token,
                        **(
                            request_options.get("additional_query_parameters", {}) or {}
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            )
        )
        if _encoded_query_params:
            ws_url = ws_url + "?" + urllib.parse.urlencode(_encoded_query_params)
        headers = self._raw_client._client_wrapper.get_headers()
        if request_options and "additional_headers" in request_options:
            headers.update(request_options["additional_headers"])
        try:
            async with websockets_client_connect(ws_url, extra_headers=headers) as protocol:
                yield AsyncControlPlaneSocketClient(websocket=protocol)
        except InvalidWebSocketStatus as exc:
            status_code: int = get_status_code(exc)
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    headers=dict(headers),
                    body="Websocket initialized with invalid credentials.",
                )
            raise ApiError(
                status_code=status_code,
                headers=dict(headers),
                body="Unexpected error when initializing websocket connection.",
            )



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from contextlib import asynccontextmanager, contextmanager

import websockets.sync.client as websockets_sync_client
from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.jsonable_encoder import jsonable_encoder
from ....core.query_encoder import encode_query
from ....core.remove_none_from_dict import remove_none_from_dict
from ....core.request_options import RequestOptions
from ....core.websocket_compat import InvalidWebSocketStatus, get_status_code
from .socket_client import AsyncStreamSocketClient, StreamSocketClient

try:
    from websockets.legacy.client import connect as websockets_client_connect  # type: ignore
except ImportError:
    from websockets import connect as websockets_client_connect  # type: ignore


class RawStreamClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    @contextmanager
    def connect(
        self, *, hume_api_key: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.Iterator[StreamSocketClient]:
        """
        Parameters
        ----------
        hume_api_key : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        StreamSocketClient
        """
        ws_url = self._client_wrapper.get_environment().stream + "/models"
        _encoded_query_params = encode_query(
            jsonable_encoder(
                remove_none_from_dict(
                    {
                        **(
                            request_options.get("additional_query_parameters", {}) or {}
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            )
        )
        if _encoded_query_params:
            ws_url = ws_url + "?" + urllib.parse.urlencode(_encoded_query_params)
        headers = self._client_wrapper.get_headers()
        if hume_api_key is not None:
            headers["X-Hume-Api-Key"] = str(hume_api_key)
        if request_options and "additional_headers" in request_options:
            headers.update(request_options["additional_headers"])
        try:
            with websockets_sync_client.connect(ws_url, additional_headers=headers) as protocol:
                yield StreamSocketClient(websocket=protocol)
        except InvalidWebSocketStatus as exc:
            status_code: int = get_status_code(exc)
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    headers=dict(headers),
                    body="Websocket initialized with invalid credentials.",
                )
            raise ApiError(
                status_code=status_code,
                headers=dict(headers),
                body="Unexpected error when initializing websocket connection.",
            )


class AsyncRawStreamClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    @asynccontextmanager
    async def connect(
        self, *, hume_api_key: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.AsyncIterator[AsyncStreamSocketClient]:
        """
        Parameters
        ----------
        hume_api_key : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncStreamSocketClient
        """
        ws_url = self._client_wrapper.get_environment().stream + "/models"
        _encoded_query_params = encode_query(
            jsonable_encoder(
                remove_none_from_dict(
                    {
                        **(
                            request_options.get("additional_query_parameters", {}) or {}
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            )
        )
        if _encoded_query_params:
            ws_url = ws_url + "?" + urllib.parse.urlencode(_encoded_query_params)
        headers = self._client_wrapper.get_headers()
        if hume_api_key is not None:
            headers["X-Hume-Api-Key"] = str(hume_api_key)
        if request_options and "additional_headers" in request_options:
            headers.update(request_options["additional_headers"])
        try:
            async with websockets_client_connect(ws_url, extra_headers=headers) as protocol:
                yield AsyncStreamSocketClient(websocket=protocol)
        except InvalidWebSocketStatus as exc:
            status_code: int = get_status_code(exc)
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    headers=dict(headers),
                    body="Websocket initialized with invalid credentials.",
                )
            raise ApiError(
                status_code=status_code,
                headers=dict(headers),
                body="Unexpected error when initializing websocket connection.",
            )



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations


class HumeClientEnvironment:
    PROD: HumeClientEnvironment

    def __init__(self, *, base: str, evi: str, tts: str, stream: str):
        self.base = base
        self.evi = evi
        self.tts = tts
        self.stream = stream


HumeClientEnvironment.PROD = HumeClientEnvironment(
    base="https://api.hume.ai",
    evi="wss://api.hume.ai/v0/evi",
    tts="wss://api.hume.ai/v0/tts",
    stream="wss://api.hume.ai/v0/stream",
)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.pagination import AsyncPager, SyncPager
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.error_response import ErrorResponse
from ..types.http_validation_error import HttpValidationError
from ..types.return_paged_voices import ReturnPagedVoices
from ..types.return_voice import ReturnVoice
from ..types.voice_provider import VoiceProvider

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawVoicesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        provider: VoiceProvider,
        page_number: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        ascending_order: typing.Optional[bool] = None,
        filter_tag: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ReturnVoice, ReturnPagedVoices]:
        """
        Lists voices you have saved in your account, or voices from the [Voice Library](https://app.hume.ai/tts/voice-library).

        Parameters
        ----------
        provider : VoiceProvider
            Specify the voice provider to filter voices returned by the endpoint:

            - **`HUME_AI`**: Lists preset, shared voices from Hume's [Voice Library](https://app.hume.ai/tts/voice-library).
            - **`CUSTOM_VOICE`**: Lists custom voices created and saved to your account.

        page_number : typing.Optional[int]
            Specifies the page number to retrieve, enabling pagination.

            This parameter uses zero-based indexing. For example, setting `page_number` to 0 retrieves the first page of results (items 0-9 if `page_size` is 10), setting `page_number` to 1 retrieves the second page (items 10-19), and so on. Defaults to 0, which retrieves the first page.

        page_size : typing.Optional[int]
            Specifies the maximum number of results to include per page, enabling pagination. The value must be between 1 and 100, inclusive.

            For example, if `page_size` is set to 10, each page will include up to 10 items. Defaults to 10.

        ascending_order : typing.Optional[bool]

        filter_tag : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ReturnVoice, ReturnPagedVoices]
            Success
        """
        page_number = page_number if page_number is not None else 0

        _response = self._client_wrapper.httpx_client.request(
            "v0/tts/voices",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "provider": provider,
                "page_number": page_number,
                "page_size": page_size,
                "ascending_order": ascending_order,
                "filter_tag": filter_tag,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ReturnPagedVoices,
                    parse_obj_as(
                        type_=ReturnPagedVoices,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.voices_page
                _has_next = True
                _get_next = lambda: self.list(
                    provider=provider,
                    page_number=page_number + 1,
                    page_size=page_size,
                    ascending_order=ascending_order,
                    filter_tag=filter_tag,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self, *, generation_id: str, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ReturnVoice]:
        """
        Saves a new custom voice to your account using the specified TTS generation ID.

        Once saved, this voice can be reused in subsequent TTS requests, ensuring consistent speech style and prosody. For more details on voice creation, see the [Voices Guide](/docs/text-to-speech-tts/voices).

        Parameters
        ----------
        generation_id : str
            A unique ID associated with this TTS generation that can be used as context for generating consistent speech style and prosody across multiple requests.

        name : str
            The name of a **Voice**.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ReturnVoice]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "v0/tts/voices",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            json={
                "generation_id": generation_id,
                "name": name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ReturnVoice,
                    parse_obj_as(
                        type_=ReturnVoice,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, *, name: str, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Deletes a previously generated custom voice.

        Parameters
        ----------
        name : str
            Name of the voice to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            "v0/tts/voices",
            base_url=self._client_wrapper.get_environment().base,
            method="DELETE",
            params={
                "name": name,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawVoicesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        provider: VoiceProvider,
        page_number: typing.Optional[int] = None,
        page_size: typing.Optional[int] = None,
        ascending_order: typing.Optional[bool] = None,
        filter_tag: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ReturnVoice, ReturnPagedVoices]:
        """
        Lists voices you have saved in your account, or voices from the [Voice Library](https://app.hume.ai/tts/voice-library).

        Parameters
        ----------
        provider : VoiceProvider
            Specify the voice provider to filter voices returned by the endpoint:

            - **`HUME_AI`**: Lists preset, shared voices from Hume's [Voice Library](https://app.hume.ai/tts/voice-library).
            - **`CUSTOM_VOICE`**: Lists custom voices created and saved to your account.

        page_number : typing.Optional[int]
            Specifies the page number to retrieve, enabling pagination.

            This parameter uses zero-based indexing. For example, setting `page_number` to 0 retrieves the first page of results (items 0-9 if `page_size` is 10), setting `page_number` to 1 retrieves the second page (items 10-19), and so on. Defaults to 0, which retrieves the first page.

        page_size : typing.Optional[int]
            Specifies the maximum number of results to include per page, enabling pagination. The value must be between 1 and 100, inclusive.

            For example, if `page_size` is set to 10, each page will include up to 10 items. Defaults to 10.

        ascending_order : typing.Optional[bool]

        filter_tag : typing.Optional[typing.Union[str, typing.Sequence[str]]]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ReturnVoice, ReturnPagedVoices]
            Success
        """
        page_number = page_number if page_number is not None else 0

        _response = await self._client_wrapper.httpx_client.request(
            "v0/tts/voices",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "provider": provider,
                "page_number": page_number,
                "page_size": page_size,
                "ascending_order": ascending_order,
                "filter_tag": filter_tag,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ReturnPagedVoices,
                    parse_obj_as(
                        type_=ReturnPagedVoices,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.voices_page
                _has_next = True

                async def _get_next():
                    return await self.list(
                        provider=provider,
                        page_number=page_number + 1,
                        page_size=page_size,
                        ascending_order=ascending_order,
                        filter_tag=filter_tag,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self, *, generation_id: str, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ReturnVoice]:
        """
        Saves a new custom voice to your account using the specified TTS generation ID.

        Once saved, this voice can be reused in subsequent TTS requests, ensuring consistent speech style and prosody. For more details on voice creation, see the [Voices Guide](/docs/text-to-speech-tts/voices).

        Parameters
        ----------
        generation_id : str
            A unique ID associated with this TTS generation that can be used as context for generating consistent speech style and prosody across multiple requests.

        name : str
            The name of a **Voice**.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ReturnVoice]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v0/tts/voices",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            json={
                "generation_id": generation_id,
                "name": name,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ReturnVoice,
                    parse_obj_as(
                        type_=ReturnVoice,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, *, name: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Deletes a previously generated custom voice.

        Parameters
        ----------
        name : str
            Name of the voice to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "v0/tts/voices",
            base_url=self._client_wrapper.get_environment().base,
            method="DELETE",
            params={
                "name": name,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorResponse,
                        parse_obj_as(
                            type_=ErrorResponse,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .return_user_defined_tool_tool_type import ReturnUserDefinedToolToolType
from .return_user_defined_tool_version_type import ReturnUserDefinedToolVersionType


class ReturnUserDefinedTool(UniversalBaseModel):
    """
    A specific tool version returned from the server
    """

    created_on: int = pydantic.Field()
    """
    The timestamp when the first version of this tool was created.
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    An optional description of what the Tool does, used by the supplemental LLM to choose when and how to call the function.
    """

    fallback_content: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional text passed to the supplemental LLM in place of the tool call result. The LLM then uses this text to generate a response back to the user, ensuring continuity in the conversation if the Tool errors.
    """

    id: str = pydantic.Field()
    """
    Identifier for a Tool. Formatted as a UUID.
    """

    modified_on: int = pydantic.Field()
    """
    The timestamp when this version of the tool was created.
    """

    name: str = pydantic.Field()
    """
    Name of the built-in tool to use. Hume supports the following built-in tools:
    
    - **web_search:** enables EVI to search the web for up-to-date information when applicable.
    - **hang_up:** closes the WebSocket connection when appropriate (e.g., after detecting a farewell in the conversation).
    
    For more information, see our guide on [using built-in tools](/docs/speech-to-speech-evi/features/tool-use#using-built-in-tools).
    """

    parameters: str = pydantic.Field()
    """
    Stringified JSON defining the parameters used by this version of the Tool.
    
    These parameters define the inputs needed for the Tool's execution, including the expected data type and description for each input field. Structured as a stringified JSON schema, this format ensures the Tool receives data in the expected format.
    """

    tool_type: ReturnUserDefinedToolToolType
    version: int = pydantic.Field()
    """
    Version number for a Tool. Version numbers should be integers. The combination of configId and version number is unique.
    """

    version_description: typing.Optional[str] = pydantic.Field(default=None)
    """
    An optional description of the Tool version.
    """

    version_type: ReturnUserDefinedToolVersionType

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .return_chat_status import ReturnChatStatus
from .return_config_spec import ReturnConfigSpec


class ReturnChat(UniversalBaseModel):
    """
    A description of chat and its status
    """

    chat_group_id: str = pydantic.Field()
    """
    Identifier for the Chat Group. Any chat resumed from this Chat will have the same `chat_group_id`. Formatted as a UUID.
    """

    config: typing.Optional[ReturnConfigSpec] = None
    end_timestamp: typing.Optional[int] = pydantic.Field(default=None)
    """
    Time at which the Chat ended. Measured in seconds since the Unix epoch.
    """

    event_count: typing.Optional[int] = pydantic.Field(default=None)
    """
    The total number of events currently in this chat.
    """

    id: str = pydantic.Field()
    """
    Identifier for a Chat. Formatted as a UUID.
    """

    metadata: typing.Optional[str] = pydantic.Field(default=None)
    """
    Stringified JSON with additional metadata about the chat.
    """

    start_timestamp: int = pydantic.Field()
    """
    Time at which the Chat started. Measured in seconds since the Unix epoch.
    """

    status: ReturnChatStatus

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .alternative import Alternative
from .custom_model_request import CustomModelRequest
from .dataset import Dataset
from .evaluation_args import EvaluationArgs
from .task import Task


class TrainingBaseRequest(UniversalBaseModel):
    custom_model: CustomModelRequest
    dataset: Dataset
    target_feature: typing.Optional[str] = None
    task: typing.Optional[Task] = None
    evaluation: typing.Optional[EvaluationArgs] = None
    alternatives: typing.Optional[typing.List[Alternative]] = None
    callback_url: typing.Optional[str] = None
    notify: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# THIS FILE IS MANUALLY MAINTAINED: see .fernignore

import typing

import httpx

from .base_client import AsyncBaseHumeClient, BaseHumeClient

from .environment import HumeClientEnvironment


def _base_url_to_environment(base_url: str) -> HumeClientEnvironment:
    if base_url.startswith("http://"):
        return HumeClientEnvironment(
            base=base_url,
            evi=base_url.replace("http://", "ws://") + "/v0/evi",
            tts=base_url.replace("http://", "ws://") + "/v0/tts",
            stream=base_url.replace("http://", "ws://") + "/v0/stream",
        )
    elif base_url.startswith("https://"):
        return HumeClientEnvironment(
            base=base_url,
            evi=base_url.replace("https://", "wss://") + "/v0/evi",
            tts=base_url.replace("https://", "wss://") + "/v0/tts",
            stream=base_url.replace("https://", "wss://") + "/v0/stream",
        )
    else:
        # Assume https if no protocol specified
        return HumeClientEnvironment(
            base="https://" + base_url,
            evi="wss://" + base_url + "/v0/evi",
            tts="wss://" + base_url + "/v0/tts",
            stream="wss://" + base_url + "/v0/stream",
        )


class HumeClient(BaseHumeClient):
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propogate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base URL to use for requests from the client. If provided, this will be converted
        to a HumeClientEnvironment. Can be a full URL (http://... or https://...) or just
        a hostname (which will default to https://).
    environment : typing.Optional[HumeClientEnvironment]
        The environment to use for requests from the client. from .environment import HumeClientEnvironment
        Defaults to None, which will use HumeClientEnvironment.PROD. Cannot be specified together with base_url.
    api_key : typing.Optional[str]
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests by default the timeout is 60 seconds, unless a custom httpx client is used, in which case a default is not set.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from hume.client import HumeClient

    client = HumeClient(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: typing.Optional[HumeClientEnvironment] = None,
        api_key: typing.Optional[str] = None,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None
    ):
        # Error if both base_url and environment are specified
        if base_url is not None and environment is not None:
            raise ValueError("Cannot specify both 'base_url' and 'environment'. Please use only one.")
        
        # Convert base_url string to environment if provided
        if base_url is not None:
            environment = _base_url_to_environment(base_url)
        
        # Default to PROD if neither base_url nor environment was provided
        if environment is None:
            environment = HumeClientEnvironment.PROD
        
        super().__init__(
            environment=environment,
            api_key=api_key,
            headers=headers,
            timeout=timeout,
            follow_redirects=follow_redirects,
            httpx_client=httpx_client,
        )


class AsyncHumeClient(AsyncBaseHumeClient):
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propogate to these functions.
    Parameters
    ----------
    base_url : typing.Optional[str]
        The base URL to use for requests from the client. If provided, this will be converted
        to a HumeClientEnvironment. Can be a full URL (http://... or https://...) or just
        a hostname (which will default to https://).
    environment : typing.Optional[HumeClientEnvironment]
        The environment to use for requests from the client. from .environment import HumeClientEnvironment
        Defaults to None, which will use HumeClientEnvironment.PROD. Cannot be specified together with base_url.
    api_key : typing.Optional[str]
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests by default the timeout is 60 seconds, unless a custom httpx client is used, in which case a default is not set.
    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.
    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.
    Examples
    --------
    from hume.client import AsyncHumeClient
    client = AsyncHumeClient(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: typing.Optional[HumeClientEnvironment] = None,
        headers: typing.Optional[typing.Dict[str, str]] = None,
        api_key: typing.Optional[str] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None
    ):
        # Error if both base_url and environment are specified
        if base_url is not None and environment is not None:
            raise ValueError("Cannot specify both 'base_url' and 'environment'. Please use only one.")
        
        # Convert base_url string to environment if provided
        if base_url is not None:
            environment = _base_url_to_environment(base_url)
        
        # Default to PROD if neither base_url nor environment was provided
        if environment is None:
            environment = HumeClientEnvironment.PROD
        
        super().__init__(
            environment=environment,
            api_key=api_key,
            headers=headers,
            timeout=timeout,
            follow_redirects=follow_redirects,
            httpx_client=httpx_client,
        )



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from .tool_call_message import ToolCallMessage
from .webhook_event_base import WebhookEventBase


class WebhookEventToolCall(WebhookEventBase):
    caller_number: typing.Optional[str] = pydantic.Field(default=None)
    """
    Phone number of the caller in E.164 format (e.g., `+12223333333`). This field is included only if the Chat was created via the [Twilio phone calling](/docs/empathic-voice-interface-evi/phone-calling) integration.
    """

    custom_session_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    User-defined session ID. Relevant only when employing a [custom language model](/docs/empathic-voice-interface-evi/custom-language-model) in the EVI Config.
    """

    event_name: typing.Optional[typing.Literal["tool_call"]] = pydantic.Field(default=None)
    """
    Always `tool_call`.
    """

    timestamp: int = pydantic.Field()
    """
    Unix timestamp (in milliseconds) indicating when the tool call was triggered.
    """

    tool_call_message: ToolCallMessage = pydantic.Field()
    """
    The tool call.
    """

    twilio_metadata: typing.Optional[typing.Dict[str, typing.Optional[str]]] = pydantic.Field(default=None)
    """
    Twilio metadata associated with the chat. This field is included only if the Chat was created via the [Twilio phone calling](/docs/empathic-voice-interface-evi/phone-calling) integration. Fields may include `call_sid`, `account_sid`, `from_number`, `to_number`, `caller_name`, `caller_number`, `from_city`, `from_state`, `from_zip`, `from_country`, `to_city`, `to_state`, `to_zip`, and `to_country`.If a specific metadata is not available, this field will be set to an empty string.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .alternative import Alternative
    from .bcp_47_tag import Bcp47Tag
    from .bounding_box import BoundingBox
    from .burst_prediction import BurstPrediction
    from .classification import Classification
    from .completed_embedding_generation import CompletedEmbeddingGeneration
    from .completed_inference import CompletedInference
    from .completed_state import CompletedState
    from .completed_tl_inference import CompletedTlInference
    from .completed_training import CompletedTraining
    from .custom_model import CustomModel
    from .custom_model_id import CustomModelId
    from .custom_model_prediction import CustomModelPrediction
    from .custom_model_request import CustomModelRequest
    from .custom_model_version_id import CustomModelVersionId
    from .custom_models_inference_job import CustomModelsInferenceJob
    from .custom_models_training_job import CustomModelsTrainingJob
    from .dataset import Dataset
    from .dataset_id import DatasetId
    from .dataset_version_id import DatasetVersionId
    from .descriptions_score import DescriptionsScore
    from .direction import Direction
    from .embedding_generation_base_request import EmbeddingGenerationBaseRequest
    from .embedding_generation_job import EmbeddingGenerationJob
    from .emotion_score import EmotionScore
    from .error import Error
    from .evaluation_args import EvaluationArgs
    from .face import Face
    from .face_prediction import FacePrediction
    from .facemesh_prediction import FacemeshPrediction
    from .facs_score import FacsScore
    from .failed import Failed
    from .failed_state import FailedState
    from .file import File
    from .granularity import Granularity
    from .grouped_predictions_burst_prediction import GroupedPredictionsBurstPrediction
    from .grouped_predictions_face_prediction import GroupedPredictionsFacePrediction
    from .grouped_predictions_facemesh_prediction import GroupedPredictionsFacemeshPrediction
    from .grouped_predictions_language_prediction import GroupedPredictionsLanguagePrediction
    from .grouped_predictions_ner_prediction import GroupedPredictionsNerPrediction
    from .grouped_predictions_prosody_prediction import GroupedPredictionsProsodyPrediction
    from .in_progress import InProgress
    from .in_progress_state import InProgressState
    from .inference_base_request import InferenceBaseRequest
    from .inference_job import InferenceJob
    from .inference_prediction import InferencePrediction
    from .inference_request import InferenceRequest
    from .inference_results import InferenceResults
    from .inference_source_predict_result import InferenceSourcePredictResult
    from .job_embedding_generation import JobEmbeddingGeneration
    from .job_id import JobId
    from .job_inference import JobInference
    from .job_tl_inference import JobTlInference
    from .job_training import JobTraining
    from .language import Language
    from .language_prediction import LanguagePrediction
    from .models import Models
    from .models_predictions import ModelsPredictions
    from .ner import Ner
    from .ner_prediction import NerPrediction
    from .null import Null
    from .position_interval import PositionInterval
    from .predictions_optional_null_burst_prediction import PredictionsOptionalNullBurstPrediction
    from .predictions_optional_null_face_prediction import PredictionsOptionalNullFacePrediction
    from .predictions_optional_null_facemesh_prediction import PredictionsOptionalNullFacemeshPrediction
    from .predictions_optional_transcription_metadata_language_prediction import (
        PredictionsOptionalTranscriptionMetadataLanguagePrediction,
    )
    from .predictions_optional_transcription_metadata_ner_prediction import (
        PredictionsOptionalTranscriptionMetadataNerPrediction,
    )
    from .predictions_optional_transcription_metadata_prosody_prediction import (
        PredictionsOptionalTranscriptionMetadataProsodyPrediction,
    )
    from .prosody import Prosody
    from .prosody_prediction import ProsodyPrediction
    from .queued import Queued
    from .queued_state import QueuedState
    from .registry_file_detail import RegistryFileDetail
    from .regression import Regression
    from .sentiment_score import SentimentScore
    from .sort_by import SortBy
    from .source import Source, Source_File, Source_Text, Source_Url
    from .source_file import SourceFile
    from .source_text_source import SourceTextSource
    from .source_url import SourceUrl
    from .state_embedding_generation import (
        StateEmbeddingGeneration,
        StateEmbeddingGeneration_Completed,
        StateEmbeddingGeneration_Failed,
        StateEmbeddingGeneration_InProgress,
        StateEmbeddingGeneration_Queued,
    )
    from .state_embedding_generation_completed_embedding_generation import (
        StateEmbeddingGenerationCompletedEmbeddingGeneration,
    )
    from .state_embedding_generation_failed import StateEmbeddingGenerationFailed
    from .state_embedding_generation_in_progress import StateEmbeddingGenerationInProgress
    from .state_embedding_generation_queued import StateEmbeddingGenerationQueued
    from .state_inference import (
        StateInference,
        StateInference_Completed,
        StateInference_Failed,
        StateInference_InProgress,
        StateInference_Queued,
    )
    from .state_tl_inference import (
        StateTlInference,
        StateTlInference_Completed,
        StateTlInference_Failed,
        StateTlInference_InProgress,
        StateTlInference_Queued,
    )
    from .state_tl_inference_completed_tl_inference import StateTlInferenceCompletedTlInference
    from .state_tl_inference_failed import StateTlInferenceFailed
    from .state_tl_inference_in_progress import StateTlInferenceInProgress
    from .state_tl_inference_queued import StateTlInferenceQueued
    from .state_training import (
        StateTraining,
        StateTraining_Completed,
        StateTraining_Failed,
        StateTraining_InProgress,
        StateTraining_Queued,
    )
    from .state_training_completed_training import StateTrainingCompletedTraining
    from .state_training_failed import StateTrainingFailed
    from .state_training_in_progress import StateTrainingInProgress
    from .state_training_queued import StateTrainingQueued
    from .status import Status
    from .tag import Tag
    from .target import Target
    from .task import Task, Task_Classification, Task_Regression
    from .task_classification import TaskClassification
    from .task_regression import TaskRegression
    from .text_source import TextSource
    from .time_interval import TimeInterval
    from .tl_inference_base_request import TlInferenceBaseRequest
    from .tl_inference_prediction import TlInferencePrediction
    from .tl_inference_results import TlInferenceResults
    from .tl_inference_source_predict_result import TlInferenceSourcePredictResult
    from .toxicity_score import ToxicityScore
    from .training_base_request import TrainingBaseRequest
    from .training_custom_model import TrainingCustomModel
    from .transcription import Transcription
    from .transcription_metadata import TranscriptionMetadata
    from .type import Type
    from .unconfigurable import Unconfigurable
    from .union_job import UnionJob
    from .union_predict_result import UnionPredictResult
    from .url import Url
    from .validation_args import ValidationArgs
    from .when import When
    from .window import Window
_dynamic_imports: typing.Dict[str, str] = {
    "Alternative": ".alternative",
    "Bcp47Tag": ".bcp_47_tag",
    "BoundingBox": ".bounding_box",
    "BurstPrediction": ".burst_prediction",
    "Classification": ".classification",
    "CompletedEmbeddingGeneration": ".completed_embedding_generation",
    "CompletedInference": ".completed_inference",
    "CompletedState": ".completed_state",
    "CompletedTlInference": ".completed_tl_inference",
    "CompletedTraining": ".completed_training",
    "CustomModel": ".custom_model",
    "CustomModelId": ".custom_model_id",
    "CustomModelPrediction": ".custom_model_prediction",
    "CustomModelRequest": ".custom_model_request",
    "CustomModelVersionId": ".custom_model_version_id",
    "CustomModelsInferenceJob": ".custom_models_inference_job",
    "CustomModelsTrainingJob": ".custom_models_training_job",
    "Dataset": ".dataset",
    "DatasetId": ".dataset_id",
    "DatasetVersionId": ".dataset_version_id",
    "DescriptionsScore": ".descriptions_score",
    "Direction": ".direction",
    "EmbeddingGenerationBaseRequest": ".embedding_generation_base_request",
    "EmbeddingGenerationJob": ".embedding_generation_job",
    "EmotionScore": ".emotion_score",
    "Error": ".error",
    "EvaluationArgs": ".evaluation_args",
    "Face": ".face",
    "FacePrediction": ".face_prediction",
    "FacemeshPrediction": ".facemesh_prediction",
    "FacsScore": ".facs_score",
    "Failed": ".failed",
    "FailedState": ".failed_state",
    "File": ".file",
    "Granularity": ".granularity",
    "GroupedPredictionsBurstPrediction": ".grouped_predictions_burst_prediction",
    "GroupedPredictionsFacePrediction": ".grouped_predictions_face_prediction",
    "GroupedPredictionsFacemeshPrediction": ".grouped_predictions_facemesh_prediction",
    "GroupedPredictionsLanguagePrediction": ".grouped_predictions_language_prediction",
    "GroupedPredictionsNerPrediction": ".grouped_predictions_ner_prediction",
    "GroupedPredictionsProsodyPrediction": ".grouped_predictions_prosody_prediction",
    "InProgress": ".in_progress",
    "InProgressState": ".in_progress_state",
    "InferenceBaseRequest": ".inference_base_request",
    "InferenceJob": ".inference_job",
    "InferencePrediction": ".inference_prediction",
    "InferenceRequest": ".inference_request",
    "InferenceResults": ".inference_results",
    "InferenceSourcePredictResult": ".inference_source_predict_result",
    "JobEmbeddingGeneration": ".job_embedding_generation",
    "JobId": ".job_id",
    "JobInference": ".job_inference",
    "JobTlInference": ".job_tl_inference",
    "JobTraining": ".job_training",
    "Language": ".language",
    "LanguagePrediction": ".language_prediction",
    "Models": ".models",
    "ModelsPredictions": ".models_predictions",
    "Ner": ".ner",
    "NerPrediction": ".ner_prediction",
    "Null": ".null",
    "PositionInterval": ".position_interval",
    "PredictionsOptionalNullBurstPrediction": ".predictions_optional_null_burst_prediction",
    "PredictionsOptionalNullFacePrediction": ".predictions_optional_null_face_prediction",
    "PredictionsOptionalNullFacemeshPrediction": ".predictions_optional_null_facemesh_prediction",
    "PredictionsOptionalTranscriptionMetadataLanguagePrediction": ".predictions_optional_transcription_metadata_language_prediction",
    "PredictionsOptionalTranscriptionMetadataNerPrediction": ".predictions_optional_transcription_metadata_ner_prediction",
    "PredictionsOptionalTranscriptionMetadataProsodyPrediction": ".predictions_optional_transcription_metadata_prosody_prediction",
    "Prosody": ".prosody",
    "ProsodyPrediction": ".prosody_prediction",
    "Queued": ".queued",
    "QueuedState": ".queued_state",
    "RegistryFileDetail": ".registry_file_detail",
    "Regression": ".regression",
    "SentimentScore": ".sentiment_score",
    "SortBy": ".sort_by",
    "Source": ".source",
    "SourceFile": ".source_file",
    "SourceTextSource": ".source_text_source",
    "SourceUrl": ".source_url",
    "Source_File": ".source",
    "Source_Text": ".source",
    "Source_Url": ".source",
    "StateEmbeddingGeneration": ".state_embedding_generation",
    "StateEmbeddingGenerationCompletedEmbeddingGeneration": ".state_embedding_generation_completed_embedding_generation",
    "StateEmbeddingGenerationFailed": ".state_embedding_generation_failed",
    "StateEmbeddingGenerationInProgress": ".state_embedding_generation_in_progress",
    "StateEmbeddingGenerationQueued": ".state_embedding_generation_queued",
    "StateEmbeddingGeneration_Completed": ".state_embedding_generation",
    "StateEmbeddingGeneration_Failed": ".state_embedding_generation",
    "StateEmbeddingGeneration_InProgress": ".state_embedding_generation",
    "StateEmbeddingGeneration_Queued": ".state_embedding_generation",
    "StateInference": ".state_inference",
    "StateInference_Completed": ".state_inference",
    "StateInference_Failed": ".state_inference",
    "StateInference_InProgress": ".state_inference",
    "StateInference_Queued": ".state_inference",
    "StateTlInference": ".state_tl_inference",
    "StateTlInferenceCompletedTlInference": ".state_tl_inference_completed_tl_inference",
    "StateTlInferenceFailed": ".state_tl_inference_failed",
    "StateTlInferenceInProgress": ".state_tl_inference_in_progress",
    "StateTlInferenceQueued": ".state_tl_inference_queued",
    "StateTlInference_Completed": ".state_tl_inference",
    "StateTlInference_Failed": ".state_tl_inference",
    "StateTlInference_InProgress": ".state_tl_inference",
    "StateTlInference_Queued": ".state_tl_inference",
    "StateTraining": ".state_training",
    "StateTrainingCompletedTraining": ".state_training_completed_training",
    "StateTrainingFailed": ".state_training_failed",
    "StateTrainingInProgress": ".state_training_in_progress",
    "StateTrainingQueued": ".state_training_queued",
    "StateTraining_Completed": ".state_training",
    "StateTraining_Failed": ".state_training",
    "StateTraining_InProgress": ".state_training",
    "StateTraining_Queued": ".state_training",
    "Status": ".status",
    "Tag": ".tag",
    "Target": ".target",
    "Task": ".task",
    "TaskClassification": ".task_classification",
    "TaskRegression": ".task_regression",
    "Task_Classification": ".task",
    "Task_Regression": ".task",
    "TextSource": ".text_source",
    "TimeInterval": ".time_interval",
    "TlInferenceBaseRequest": ".tl_inference_base_request",
    "TlInferencePrediction": ".tl_inference_prediction",
    "TlInferenceResults": ".tl_inference_results",
    "TlInferenceSourcePredictResult": ".tl_inference_source_predict_result",
    "ToxicityScore": ".toxicity_score",
    "TrainingBaseRequest": ".training_base_request",
    "TrainingCustomModel": ".training_custom_model",
    "Transcription": ".transcription",
    "TranscriptionMetadata": ".transcription_metadata",
    "Type": ".type",
    "Unconfigurable": ".unconfigurable",
    "UnionJob": ".union_job",
    "UnionPredictResult": ".union_predict_result",
    "Url": ".url",
    "ValidationArgs": ".validation_args",
    "When": ".when",
    "Window": ".window",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Alternative",
    "Bcp47Tag",
    "BoundingBox",
    "BurstPrediction",
    "Classification",
    "CompletedEmbeddingGeneration",
    "CompletedInference",
    "CompletedState",
    "CompletedTlInference",
    "CompletedTraining",
    "CustomModel",
    "CustomModelId",
    "CustomModelPrediction",
    "CustomModelRequest",
    "CustomModelVersionId",
    "CustomModelsInferenceJob",
    "CustomModelsTrainingJob",
    "Dataset",
    "DatasetId",
    "DatasetVersionId",
    "DescriptionsScore",
    "Direction",
    "EmbeddingGenerationBaseRequest",
    "EmbeddingGenerationJob",
    "EmotionScore",
    "Error",
    "EvaluationArgs",
    "Face",
    "FacePrediction",
    "FacemeshPrediction",
    "FacsScore",
    "Failed",
    "FailedState",
    "File",
    "Granularity",
    "GroupedPredictionsBurstPrediction",
    "GroupedPredictionsFacePrediction",
    "GroupedPredictionsFacemeshPrediction",
    "GroupedPredictionsLanguagePrediction",
    "GroupedPredictionsNerPrediction",
    "GroupedPredictionsProsodyPrediction",
    "InProgress",
    "InProgressState",
    "InferenceBaseRequest",
    "InferenceJob",
    "InferencePrediction",
    "InferenceRequest",
    "InferenceResults",
    "InferenceSourcePredictResult",
    "JobEmbeddingGeneration",
    "JobId",
    "JobInference",
    "JobTlInference",
    "JobTraining",
    "Language",
    "LanguagePrediction",
    "Models",
    "ModelsPredictions",
    "Ner",
    "NerPrediction",
    "Null",
    "PositionInterval",
    "PredictionsOptionalNullBurstPrediction",
    "PredictionsOptionalNullFacePrediction",
    "PredictionsOptionalNullFacemeshPrediction",
    "PredictionsOptionalTranscriptionMetadataLanguagePrediction",
    "PredictionsOptionalTranscriptionMetadataNerPrediction",
    "PredictionsOptionalTranscriptionMetadataProsodyPrediction",
    "Prosody",
    "ProsodyPrediction",
    "Queued",
    "QueuedState",
    "RegistryFileDetail",
    "Regression",
    "SentimentScore",
    "SortBy",
    "Source",
    "SourceFile",
    "SourceTextSource",
    "SourceUrl",
    "Source_File",
    "Source_Text",
    "Source_Url",
    "StateEmbeddingGeneration",
    "StateEmbeddingGenerationCompletedEmbeddingGeneration",
    "StateEmbeddingGenerationFailed",
    "StateEmbeddingGenerationInProgress",
    "StateEmbeddingGenerationQueued",
    "StateEmbeddingGeneration_Completed",
    "StateEmbeddingGeneration_Failed",
    "StateEmbeddingGeneration_InProgress",
    "StateEmbeddingGeneration_Queued",
    "StateInference",
    "StateInference_Completed",
    "StateInference_Failed",
    "StateInference_InProgress",
    "StateInference_Queued",
    "StateTlInference",
    "StateTlInferenceCompletedTlInference",
    "StateTlInferenceFailed",
    "StateTlInferenceInProgress",
    "StateTlInferenceQueued",
    "StateTlInference_Completed",
    "StateTlInference_Failed",
    "StateTlInference_InProgress",
    "StateTlInference_Queued",
    "StateTraining",
    "StateTrainingCompletedTraining",
    "StateTrainingFailed",
    "StateTrainingInProgress",
    "StateTrainingQueued",
    "StateTraining_Completed",
    "StateTraining_Failed",
    "StateTraining_InProgress",
    "StateTraining_Queued",
    "Status",
    "Tag",
    "Target",
    "Task",
    "TaskClassification",
    "TaskRegression",
    "Task_Classification",
    "Task_Regression",
    "TextSource",
    "TimeInterval",
    "TlInferenceBaseRequest",
    "TlInferencePrediction",
    "TlInferenceResults",
    "TlInferenceSourcePredictResult",
    "ToxicityScore",
    "TrainingBaseRequest",
    "TrainingCustomModel",
    "Transcription",
    "TranscriptionMetadata",
    "Type",
    "Unconfigurable",
    "UnionJob",
    "UnionPredictResult",
    "Url",
    "ValidationArgs",
    "When",
    "Window",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .chat_message import ChatMessage
from .inference import Inference
from .millisecond_interval import MillisecondInterval


class UserMessage(UniversalBaseModel):
    """
    **Transcript of the user's message.** Contains the message role and content, along with a `from_text` field indicating if this message was inserted into the conversation as text from a `UserInput` message.

    Includes an `interim` field indicating whether the transcript is provisional (words may be repeated or refined in subsequent `UserMessage` responses as additional audio is processed) or final and complete. Interim transcripts are only sent when the `verbose_transcription` query parameter is set to true in the initial handshake.
    """

    custom_session_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Used to manage conversational state, correlate frontend and backend data, and persist conversations across EVI sessions.
    """

    from_text: bool = pydantic.Field()
    """
    Indicates if this message was inserted into the conversation as text from a [User Input](/reference/empathic-voice-interface-evi/chat/chat#send.User%20Input.text) message.
    """

    interim: bool = pydantic.Field()
    """
    Indicates if this message contains an immediate and unfinalized transcript of the user's audio input. If it does, words may be repeated across successive UserMessage messages as our transcription model becomes more confident about what was said with additional context. Interim messages are useful to detect if the user is interrupting during audio playback on the client. Even without a finalized transcription, along with `UserInterrupt` messages, interim `UserMessages` are useful for detecting if the user is interrupting during audio playback on the client, signaling to stop playback in your application.
    """

    language: typing.Optional[str] = pydantic.Field(default=None)
    """
    Detected language of the message text.
    """

    message: ChatMessage = pydantic.Field()
    """
    Transcript of the message.
    """

    models: Inference = pydantic.Field()
    """
    Inference model results.
    """

    time: MillisecondInterval = pydantic.Field()
    """
    Start and End time of user message.
    """

    type: typing.Literal["user_message"] = "user_message"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .posted_event_message_spec import PostedEventMessageSpec


class PostedEventMessageSpecs(UniversalBaseModel):
    """
    Collection of event messages returned by the server.

    Event messages are sent by the server when specific events occur during a chat session. These messages are used to configure behaviors for EVI, such as controlling how EVI starts a new conversation.
    """

    on_inactivity_timeout: typing.Optional[PostedEventMessageSpec] = None
    on_max_duration_timeout: typing.Optional[PostedEventMessageSpec] = None
    on_new_chat: typing.Optional[PostedEventMessageSpec] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from contextlib import asynccontextmanager, contextmanager

import websockets.sync.client as websockets_sync_client
from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import jsonable_encoder
from ...core.query_encoder import encode_query
from ...core.remove_none_from_dict import remove_none_from_dict
from ...core.request_options import RequestOptions
from ...core.websocket_compat import InvalidWebSocketStatus, get_status_code
from ..types.audio_format_type import AudioFormatType
from ..types.octave_version import OctaveVersion
from ..types.timestamp_type import TimestampType
from .raw_client import AsyncRawStreamInputClient, RawStreamInputClient
from .socket_client import AsyncStreamInputSocketClient, StreamInputSocketClient

try:
    from websockets.legacy.client import connect as websockets_client_connect  # type: ignore
except ImportError:
    from websockets import connect as websockets_client_connect  # type: ignore


class StreamInputClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawStreamInputClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawStreamInputClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawStreamInputClient
        """
        return self._raw_client

    @contextmanager
    def connect(
        self,
        *,
        access_token: typing.Optional[str] = None,
        context_generation_id: typing.Optional[str] = None,
        format_type: typing.Optional[AudioFormatType] = None,
        include_timestamp_types: typing.Optional[typing.Union[TimestampType, typing.Sequence[TimestampType]]] = None,
        instant_mode: typing.Optional[bool] = None,
        no_binary: typing.Optional[bool] = None,
        strip_headers: typing.Optional[bool] = None,
        version: typing.Optional[OctaveVersion] = None,
        api_key: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[StreamInputSocketClient]:
        """
        Generate emotionally expressive speech.

        Parameters
        ----------
        access_token : typing.Optional[str]
            Access token used for authenticating the client. If not provided, an `api_key` must be provided to authenticate.

            The access token is generated using both an API key and a Secret key, which provides an additional layer of security compared to using just an API key.

            For more details, refer to the [Authentication Strategies Guide](/docs/introduction/api-key#authentication-strategies).

        context_generation_id : typing.Optional[str]
            The ID of a prior TTS generation to use as context for generating consistent speech style and prosody across multiple requests. Including context may increase audio generation times.

        format_type : typing.Optional[AudioFormatType]
            The format to be used for audio generation.

        include_timestamp_types : typing.Optional[typing.Union[TimestampType, typing.Sequence[TimestampType]]]
            The set of timestamp types to include in the response. Only supported for Octave 2 requests.

        instant_mode : typing.Optional[bool]
            Enables ultra-low latency streaming, significantly reducing the time until the first audio chunk is received. Recommended for real-time applications requiring immediate audio playback. For further details, see our documentation on [instant mode](/docs/text-to-speech-tts/overview#ultra-low-latency-streaming-instant-mode).

        no_binary : typing.Optional[bool]
            If enabled, no binary websocket messages will be sent to the client.

        strip_headers : typing.Optional[bool]
            If enabled, the audio for all the chunks of a generation, once concatenated together, will constitute a single audio file. Otherwise, if disabled, each chunk's audio will be its own audio file, each with its own headers (if applicable).

        version : typing.Optional[OctaveVersion]
            The version of the Octave Model to use. 1 for the legacy model, 2 for the new model.

        api_key : typing.Optional[str]
            API key used for authenticating the client. If not provided, an `access_token` must be provided to authenticate.

            For more details, refer to the [Authentication Strategies Guide](/docs/introduction/api-key#authentication-strategies).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        StreamInputSocketClient
        """
        ws_url = self._raw_client._client_wrapper.get_environment().tts + "/stream/input"
        _encoded_query_params = encode_query(
            jsonable_encoder(
                remove_none_from_dict(
                    {
                        "access_token": access_token,
                        "context_generation_id": context_generation_id,
                        "format_type": format_type,
                        "include_timestamp_types": include_timestamp_types,
                        "instant_mode": instant_mode,
                        "no_binary": no_binary,
                        "strip_headers": strip_headers,
                        "version": version,
                        "api_key": api_key,
                        **(
                            request_options.get("additional_query_parameters", {}) or {}
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            )
        )
        if _encoded_query_params:
            ws_url = ws_url + "?" + urllib.parse.urlencode(_encoded_query_params)
        headers = self._raw_client._client_wrapper.get_headers()
        if request_options and "additional_headers" in request_options:
            headers.update(request_options["additional_headers"])
        try:
            with websockets_sync_client.connect(ws_url, additional_headers=headers) as protocol:
                yield StreamInputSocketClient(websocket=protocol)
        except InvalidWebSocketStatus as exc:
            status_code: int = get_status_code(exc)
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    headers=dict(headers),
                    body="Websocket initialized with invalid credentials.",
                )
            raise ApiError(
                status_code=status_code,
                headers=dict(headers),
                body="Unexpected error when initializing websocket connection.",
            )


class AsyncStreamInputClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawStreamInputClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawStreamInputClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawStreamInputClient
        """
        return self._raw_client

    @asynccontextmanager
    async def connect(
        self,
        *,
        access_token: typing.Optional[str] = None,
        context_generation_id: typing.Optional[str] = None,
        format_type: typing.Optional[AudioFormatType] = None,
        include_timestamp_types: typing.Optional[typing.Union[TimestampType, typing.Sequence[TimestampType]]] = None,
        instant_mode: typing.Optional[bool] = None,
        no_binary: typing.Optional[bool] = None,
        strip_headers: typing.Optional[bool] = None,
        version: typing.Optional[OctaveVersion] = None,
        api_key: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncStreamInputSocketClient]:
        """
        Generate emotionally expressive speech.

        Parameters
        ----------
        access_token : typing.Optional[str]
            Access token used for authenticating the client. If not provided, an `api_key` must be provided to authenticate.

            The access token is generated using both an API key and a Secret key, which provides an additional layer of security compared to using just an API key.

            For more details, refer to the [Authentication Strategies Guide](/docs/introduction/api-key#authentication-strategies).

        context_generation_id : typing.Optional[str]
            The ID of a prior TTS generation to use as context for generating consistent speech style and prosody across multiple requests. Including context may increase audio generation times.

        format_type : typing.Optional[AudioFormatType]
            The format to be used for audio generation.

        include_timestamp_types : typing.Optional[typing.Union[TimestampType, typing.Sequence[TimestampType]]]
            The set of timestamp types to include in the response. Only supported for Octave 2 requests.

        instant_mode : typing.Optional[bool]
            Enables ultra-low latency streaming, significantly reducing the time until the first audio chunk is received. Recommended for real-time applications requiring immediate audio playback. For further details, see our documentation on [instant mode](/docs/text-to-speech-tts/overview#ultra-low-latency-streaming-instant-mode).

        no_binary : typing.Optional[bool]
            If enabled, no binary websocket messages will be sent to the client.

        strip_headers : typing.Optional[bool]
            If enabled, the audio for all the chunks of a generation, once concatenated together, will constitute a single audio file. Otherwise, if disabled, each chunk's audio will be its own audio file, each with its own headers (if applicable).

        version : typing.Optional[OctaveVersion]
            The version of the Octave Model to use. 1 for the legacy model, 2 for the new model.

        api_key : typing.Optional[str]
            API key used for authenticating the client. If not provided, an `access_token` must be provided to authenticate.

            For more details, refer to the [Authentication Strategies Guide](/docs/introduction/api-key#authentication-strategies).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncStreamInputSocketClient
        """
        ws_url = self._raw_client._client_wrapper.get_environment().tts + "/stream/input"
        _encoded_query_params = encode_query(
            jsonable_encoder(
                remove_none_from_dict(
                    {
                        "access_token": access_token,
                        "context_generation_id": context_generation_id,
                        "format_type": format_type,
                        "include_timestamp_types": include_timestamp_types,
                        "instant_mode": instant_mode,
                        "no_binary": no_binary,
                        "strip_headers": strip_headers,
                        "version": version,
                        "api_key": api_key,
                        **(
                            request_options.get("additional_query_parameters", {}) or {}
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            )
        )
        if _encoded_query_params:
            ws_url = ws_url + "?" + urllib.parse.urlencode(_encoded_query_params)
        headers = self._raw_client._client_wrapper.get_headers()
        if request_options and "additional_headers" in request_options:
            headers.update(request_options["additional_headers"])
        try:
            async with websockets_client_connect(ws_url, extra_headers=headers) as protocol:
                yield AsyncStreamInputSocketClient(websocket=protocol)
        except InvalidWebSocketStatus as exc:
            status_code: int = get_status_code(exc)
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    headers=dict(headers),
                    body="Websocket initialized with invalid credentials.",
                )
            raise ApiError(
                status_code=status_code,
                headers=dict(headers),
                body="Unexpected error when initializing websocket connection.",
            )



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .format import Format
from .octave_version import OctaveVersion
from .posted_context import PostedContext
from .posted_utterance import PostedUtterance
from .timestamp_type import TimestampType


class PostedTts(UniversalBaseModel):
    context: typing.Optional[PostedContext] = pydantic.Field(default=None)
    """
    Utterances to use as context for generating consistent speech style and prosody across multiple requests. These will not be converted to speech output.
    """

    format: typing.Optional[Format] = pydantic.Field(default=None)
    """
    Specifies the output audio file format.
    """

    include_timestamp_types: typing.Optional[typing.List[TimestampType]] = pydantic.Field(default=None)
    """
    The set of timestamp types to include in the response. Only supported for Octave 2 requests.
    """

    num_generations: typing.Optional[int] = pydantic.Field(default=None)
    """
    Number of audio generations to produce from the input utterances.
    
    Using `num_generations` enables faster processing than issuing multiple sequential requests. Additionally, specifying `num_generations` allows prosody continuation across all generations without repeating context, ensuring each generation sounds slightly different while maintaining contextual consistency.
    """

    split_utterances: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Controls how audio output is segmented in the response.
    
    - When **enabled** (`true`), input utterances are automatically split into natural-sounding speech segments.
    
    - When **disabled** (`false`), the response maintains a strict one-to-one mapping between input utterances and output snippets. 
    
    This setting affects how the `snippets` array is structured in the response, which may be important for applications that need to track the relationship between input text and generated audio segments. When setting to `false`, avoid including utterances with long `text`, as this can result in distorted output.
    """

    strip_headers: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If enabled, the audio for all the chunks of a generation, once concatenated together, will constitute a single audio file. Otherwise, if disabled, each chunk's audio will be its own audio file, each with its own headers (if applicable).
    """

    utterances: typing.List[PostedUtterance] = pydantic.Field()
    """
    A list of **Utterances** to be converted to speech output.
    
    An **Utterance** is a unit of input for [Octave](/docs/text-to-speech-tts/overview), and includes input `text`, an optional `description` to serve as the prompt for how the speech should be delivered, an optional `voice` specification, and additional controls to guide delivery for `speed` and `trailing_silence`.
    """

    version: typing.Optional[OctaveVersion] = pydantic.Field(default=None)
    """
    Selects the Octave model version used to synthesize speech for this request. If you omit this field, Hume automatically routes the request to the most appropriate model. Setting a specific version ensures stable and repeatable behavior across requests.
    
    Use `2` to opt into the latest Octave capabilities. When you specify version `2`, you must also provide a `voice`. Requests that set `version: 2` without a voice will be rejected.
    
    For a comparison of Octave versions, see the [Octave versions](/docs/text-to-speech-tts/overview#octave-versions) section in the TTS overview.
    """

    instant_mode: typing.Optional[bool] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .audio_configuration import AudioConfiguration
from .builtin_tool_config import BuiltinToolConfig
from .context import Context
from .session_settings_variables_value import SessionSettingsVariablesValue
from .tool import Tool


class SessionSettings(UniversalBaseModel):
    """
    **Settings for this chat session.** Session settings are temporary and apply only to the current Chat session.

    These settings can be adjusted dynamically based on the requirements of each session to ensure optimal performance and user experience. See our [Session Settings Guide](/docs/speech-to-speech-evi/configuration/session-settings) for a complete list of configurable settings.
    """

    audio: typing.Optional[AudioConfiguration] = pydantic.Field(default=None)
    """
    Configuration details for the audio input used during the session. Ensures the audio is being correctly set up for processing.
    
    This optional field is only required when the audio input is encoded in PCM Linear 16 (16-bit, little-endian, signed PCM WAV data). For detailed instructions on how to configure session settings for PCM Linear 16 audio, please refer to the [Session Settings section](/docs/empathic-voice-interface-evi/configuration#session-settings) on the EVI Configuration page.
    """

    builtin_tools: typing.Optional[typing.List[BuiltinToolConfig]] = pydantic.Field(default=None)
    """
    List of built-in tools to enable for the session.
    
    Tools are resources used by EVI to perform various tasks, such as searching the web or calling external APIs. Built-in tools, like web search, are natively integrated, while user-defined tools are created and invoked by the user. To learn more, see our [Tool Use Guide](/docs/empathic-voice-interface-evi/tool-use).
    
    Currently, the only built-in tool Hume provides is **Web Search**. When enabled, Web Search equips EVI with the ability to search the web for up-to-date information.
    """

    context: typing.Optional[Context] = pydantic.Field(default=None)
    """
    Field for injecting additional context into the conversation, which is appended to the end of user messages for the session.
    
    When included in a Session Settings message, the provided context can be used to remind the LLM of its role in every user message, prevent it from forgetting important details, or add new relevant information to the conversation.
    
    Set to `null` to clear injected context.
    """

    custom_session_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the session. Used to manage conversational state, correlate frontend and backend data, and persist conversations across EVI sessions.
    
    If included, the response sent from Hume to your backend will include this ID. This allows you to correlate frontend users with their incoming messages.
    
    It is recommended to pass a `custom_session_id` if you are using a Custom Language Model. Please see our guide to [using a custom language model](/docs/empathic-voice-interface-evi/custom-language-model) with EVI to learn more.
    """

    language_model_api_key: typing.Optional[str] = pydantic.Field(default=None)
    """
    Third party API key for the supplemental language model.
    
    When provided, EVI will use this key instead of Hume's API key for the supplemental LLM. This allows you to bypass rate limits and utilize your own API key as needed.
    """

    metadata: typing.Optional[typing.Dict[str, typing.Any]] = None
    system_prompt: typing.Optional[str] = pydantic.Field(default=None)
    """
    Instructions used to shape EVI's behavior, responses, and style for the session.
    
    When included in a Session Settings message, the provided Prompt overrides the existing one specified in the EVI configuration. If no Prompt was defined in the configuration, this Prompt will be the one used for the session.
    
    You can use the Prompt to define a specific goal or role for EVI, specifying how it should act or what it should focus on during the conversation. For example, EVI can be instructed to act as a customer support representative, a fitness coach, or a travel advisor, each with its own set of behaviors and response styles.
    
    For help writing a system prompt, see our [Prompting Guide](/docs/empathic-voice-interface-evi/prompting).
    """

    tools: typing.Optional[typing.List[Tool]] = pydantic.Field(default=None)
    """
    List of user-defined tools to enable for the session.
    
    Tools are resources used by EVI to perform various tasks, such as searching the web or calling external APIs. Built-in tools, like web search, are natively integrated, while user-defined tools are created and invoked by the user. To learn more, see our [Tool Use Guide](/docs/empathic-voice-interface-evi/tool-use).
    """

    type: typing.Literal["session_settings"] = pydantic.Field(default="session_settings")
    """
    The type of message sent through the socket; must be `session_settings` for our server to correctly identify and process it as a Session Settings message.
    
    Session settings are temporary and apply only to the current Chat session. These settings can be adjusted dynamically based on the requirements of each session to ensure optimal performance and user experience.
    
    For more information, please refer to the [Session Settings section](/docs/empathic-voice-interface-evi/configuration#session-settings) on the EVI Configuration page.
    """

    variables: typing.Optional[typing.Dict[str, SessionSettingsVariablesValue]] = pydantic.Field(default=None)
    """
    This field allows you to assign values to dynamic variables referenced in your system prompt.
    
    Each key represents the variable name, and the corresponding value is the specific content you wish to assign to that variable within the session. While the values for variables can be strings, numbers, or booleans, the value will ultimately be converted to a string when injected into your system prompt.
    
    Using this field, you can personalize responses based on session-specific details. For more guidance, see our [guide on using dynamic variables](/docs/speech-to-speech-evi/features/dynamic-variables).
    """

    voice_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Allows you to change the voice during an active chat. Updating the voice does not affect chat context or conversation history.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .context_type import ContextType


class ConnectSessionSettingsContext(UniversalBaseModel):
    """
    Field for injecting additional context into the conversation, which is appended to the end of user messages for the session.

    When included in a Session Settings message, the provided context can be used to remind the LLM of its role in every user message, prevent it from forgetting important details, or add new relevant information to the conversation.

    Set to `null` to clear injected context.
    """

    text: typing.Optional[str] = pydantic.Field(default=None)
    """
    The context to be injected into the conversation. Helps inform the LLM's response by providing relevant information about the ongoing conversation.
    
    This text will be appended to the end of [user_messages](/reference/speech-to-speech-evi/chat#receive.UserMessage.message.content) based on the chosen persistence level. For example, if you want to remind EVI of its role as a helpful weather assistant, the context you insert will be appended to the end of user messages as `{Context: You are a helpful weather assistant}`.
    """

    type: typing.Optional[ContextType] = pydantic.Field(default=None)
    """
    The persistence level of the injected context. Specifies how long the injected context will remain active in the session.
    
    - **Temporary**: Context that is only applied to the following assistant response.
    
    - **Persistent**: Context that is applied to all subsequent assistant responses for the remainder of the Chat.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

LanguageModelType = typing.Union[
    typing.Literal[
        "claude-3-7-sonnet-latest",
        "claude-3-5-sonnet-latest",
        "claude-3-5-haiku-latest",
        "claude-3-5-sonnet-20240620",
        "claude-3-opus-20240229",
        "claude-3-sonnet-20240229",
        "claude-3-haiku-20240307",
        "claude-sonnet-4-20250514",
        "claude-sonnet-4-5-20250929",
        "claude-haiku-4-5-20251001",
        "us.anthropic.claude-3-5-haiku-20241022-v1:0",
        "us.anthropic.claude-3-5-sonnet-20240620-v1:0",
        "us.anthropic.claude-3-haiku-20240307-v1:0",
        "gpt-oss-120b",
        "qwen-3-235b-a22b",
        "qwen-3-235b-a22b-instruct-2507",
        "qwen-3-235b-a22b-thinking-2507",
        "gemini-1.5-pro",
        "gemini-1.5-flash",
        "gemini-1.5-pro-002",
        "gemini-1.5-flash-002",
        "gemini-2.0-flash",
        "gemini-2.5-flash",
        "gemini-2.5-flash-preview-04-17",
        "gpt-4-turbo",
        "gpt-4-turbo-preview",
        "gpt-3.5-turbo-0125",
        "gpt-3.5-turbo",
        "gpt-4o",
        "gpt-4o-mini",
        "gpt-4.1",
        "gpt-5",
        "gpt-5-mini",
        "gpt-5-nano",
        "gpt-4o-priority",
        "gpt-4o-mini-priority",
        "gpt-4.1-priority",
        "gpt-5-priority",
        "gpt-5-mini-priority",
        "gpt-5-nano-priority",
        "gemma-7b-it",
        "llama3-8b-8192",
        "llama3-70b-8192",
        "llama-3.1-70b-versatile",
        "llama-3.3-70b-versatile",
        "llama-3.1-8b-instant",
        "moonshotai/kimi-k2-instruct",
        "accounts/fireworks/models/mixtral-8x7b-instruct",
        "accounts/fireworks/models/llama-v3p1-405b-instruct",
        "accounts/fireworks/models/llama-v3p1-70b-instruct",
        "accounts/fireworks/models/llama-v3p1-8b-instruct",
        "sonar",
        "sonar-pro",
        "sambanova",
        "DeepSeek-R1-Distill-Llama-70B",
        "Llama-4-Maverick-17B-128E-Instruct",
        "Qwen3-32B",
        "grok-4-fast-non-reasoning-latest",
        "ellm",
        "custom-language-model",
        "hume-evi-3-web-search",
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from .....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...types.emotion_embedding import EmotionEmbedding
from ...types.stream_bounding_box import StreamBoundingBox


class StreamModelPredictionsFacePredictionsItem(UniversalBaseModel):
    frame: typing.Optional[float] = pydantic.Field(default=None)
    """
    Frame number
    """

    time: typing.Optional[float] = pydantic.Field(default=None)
    """
    Time in seconds when face detection occurred.
    """

    bbox: typing.Optional[StreamBoundingBox] = None
    prob: typing.Optional[float] = pydantic.Field(default=None)
    """
    The predicted probability that a detected face was actually a face.
    """

    face_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Identifier for a face. Not that this defaults to `unknown` unless face identification is enabled in the face model configuration.
    """

    emotions: typing.Optional[EmotionEmbedding] = None
    facs: typing.Optional[EmotionEmbedding] = None
    descriptions: typing.Optional[EmotionEmbedding] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from contextlib import asynccontextmanager, contextmanager
from json.decoder import JSONDecodeError

import websockets.sync.client as websockets_sync_client
from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.pydantic_utilities import parse_obj_as
from ...core.query_encoder import encode_query
from ...core.remove_none_from_dict import remove_none_from_dict
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...core.websocket_compat import InvalidWebSocketStatus, get_status_code
from ..errors.unprocessable_entity_error import UnprocessableEntityError
from ..types.control_plane_publish_event import ControlPlanePublishEvent
from ..types.http_validation_error import HttpValidationError
from .socket_client import AsyncControlPlaneSocketClient, ControlPlaneSocketClient

try:
    from websockets.legacy.client import connect as websockets_client_connect  # type: ignore
except ImportError:
    from websockets import connect as websockets_client_connect  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawControlPlaneClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def send(
        self,
        chat_id: str,
        *,
        request: ControlPlanePublishEvent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[None]:
        """
        Send a message to a specific chat.

        Parameters
        ----------
        chat_id : str

        request : ControlPlanePublishEvent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"v0/evi/chat/{jsonable_encoder(chat_id)}/send",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=ControlPlanePublishEvent, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    @contextmanager
    def connect(
        self,
        chat_id: str,
        *,
        access_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[ControlPlaneSocketClient]:
        """
        Connects to an in-progress EVI chat session. The original chat must have been started with `allow_connection=true`. The connection can be used to send and receive the same messages as the original chat, with the exception that `audio_input` messages are not allowed.

        Parameters
        ----------
        chat_id : str
            The ID of the chat to connect to.

        access_token : typing.Optional[str]
            Access token used for authenticating the client. If not provided, an `api_key` must be provided to authenticate.

            The access token is generated using both an API key and a Secret key, which provides an additional layer of security compared to using just an API key.

            For more details, refer to the [Authentication Strategies Guide](/docs/introduction/api-key#authentication-strategies).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ControlPlaneSocketClient
        """
        ws_url = self._client_wrapper.get_environment().evi + "/chat/"
        _encoded_query_params = encode_query(
            jsonable_encoder(
                remove_none_from_dict(
                    {
                        "access_token": access_token,
                        **(
                            request_options.get("additional_query_parameters", {}) or {}
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            )
        )
        if _encoded_query_params:
            ws_url = ws_url + "?" + urllib.parse.urlencode(_encoded_query_params)
        headers = self._client_wrapper.get_headers()
        if request_options and "additional_headers" in request_options:
            headers.update(request_options["additional_headers"])
        try:
            with websockets_sync_client.connect(ws_url, additional_headers=headers) as protocol:
                yield ControlPlaneSocketClient(websocket=protocol)
        except InvalidWebSocketStatus as exc:
            status_code: int = get_status_code(exc)
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    headers=dict(headers),
                    body="Websocket initialized with invalid credentials.",
                )
            raise ApiError(
                status_code=status_code,
                headers=dict(headers),
                body="Unexpected error when initializing websocket connection.",
            )


class AsyncRawControlPlaneClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def send(
        self,
        chat_id: str,
        *,
        request: ControlPlanePublishEvent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[None]:
        """
        Send a message to a specific chat.

        Parameters
        ----------
        chat_id : str

        request : ControlPlanePublishEvent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"v0/evi/chat/{jsonable_encoder(chat_id)}/send",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            json=convert_and_respect_annotation_metadata(
                object_=request, annotation=ControlPlanePublishEvent, direction="write"
            ),
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    @asynccontextmanager
    async def connect(
        self,
        chat_id: str,
        *,
        access_token: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncControlPlaneSocketClient]:
        """
        Connects to an in-progress EVI chat session. The original chat must have been started with `allow_connection=true`. The connection can be used to send and receive the same messages as the original chat, with the exception that `audio_input` messages are not allowed.

        Parameters
        ----------
        chat_id : str
            The ID of the chat to connect to.

        access_token : typing.Optional[str]
            Access token used for authenticating the client. If not provided, an `api_key` must be provided to authenticate.

            The access token is generated using both an API key and a Secret key, which provides an additional layer of security compared to using just an API key.

            For more details, refer to the [Authentication Strategies Guide](/docs/introduction/api-key#authentication-strategies).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncControlPlaneSocketClient
        """
        ws_url = self._client_wrapper.get_environment().evi + "/chat/"
        _encoded_query_params = encode_query(
            jsonable_encoder(
                remove_none_from_dict(
                    {
                        "access_token": access_token,
                        **(
                            request_options.get("additional_query_parameters", {}) or {}
                            if request_options is not None
                            else {}
                        ),
                    }
                )
            )
        )
        if _encoded_query_params:
            ws_url = ws_url + "?" + urllib.parse.urlencode(_encoded_query_params)
        headers = self._client_wrapper.get_headers()
        if request_options and "additional_headers" in request_options:
            headers.update(request_options["additional_headers"])
        try:
            async with websockets_client_connect(ws_url, extra_headers=headers) as protocol:
                yield AsyncControlPlaneSocketClient(websocket=protocol)
        except InvalidWebSocketStatus as exc:
            status_code: int = get_status_code(exc)
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    headers=dict(headers),
                    body="Websocket initialized with invalid credentials.",
                )
            raise ApiError(
                status_code=status_code,
                headers=dict(headers),
                body="Unexpected error when initializing websocket connection.",
            )



# This file was auto-generated by Fern from our API Definition.

import typing

ReturnChatEventType = typing.Union[
    typing.Literal[
        "FUNCTION_CALL",
        "FUNCTION_CALL_RESPONSE",
        "CHAT_END_MESSAGE",
        "AGENT_MESSAGE",
        "SYSTEM_PROMPT",
        "USER_RECORDING_START_MESSAGE",
        "RESUME_ONSET",
        "USER_INTERRUPTION",
        "CHAT_START_MESSAGE",
        "PAUSE_ONSET",
        "USER_MESSAGE",
    ],
    typing.Any,
]
